import numpy as np
import astropy.convolution as apc
import vortexa_mod_randomwalk as mf18
import mylcgen as mlc
import myresample as mrs
import itertools
import matplotlib.gridspec as gridspec
import matplotlib.pylab as plt
import pandas as pd
import vortexa_convert_df2lc as c2l
import vortexa_polyfit as vpf
import matplotlib.gridspec as gridspec
import vortexa_rw_plus_poly as vrwp
import vortexa_rw_simple as vrs
import scipy.signal as ss


#make syntax correct for data frame input
def mass_mod_test(meta_df_in,dateref=-1,forecast_period = 90,figure_title='',model='poly',
convolve=None,transform_model='same',labels=[],dtgrid=10.0,normalise=1,polyorder=1,
accuracy_check=10,custom_freqs = [],figure_label = ''):


 #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 #!!!!!!!!!!!!!!!!!! Data preparation !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!



#4/12/2018 - incorporate accuracy check
#if accuracy check > 0then first fit model to ndat - 10 points and see how accurate the model is
#two metrics 
#1) how many points within uncertainty envelopes as percentage
#2) what is the model variance
 accuracy_envelope = []
 accuracy_variance = []
 accuracy_mape     = []


 nframes = len(meta_df_in)
 color   = ['k','r','b','cyan','purple','green']*nframes

 #identify earliest date from all the time axis and use this as the first time by default
 datemax_save = []
 meta_df = []
 if (dateref == -1):
  dates = pd.concat([meta_df_in[i].iloc[:,0] for i in range(nframes)])
  datemax_save.append([meta_df_in[i].iloc[-1,0] for i in range(nframes)])
  datemin = min(dates)
  datemax = max(dates) + pd.Timedelta(days=forecast_period)
  date_trunc = np.min(np.array(datemax_save))
  print ('Earliest dat in sample...',datemin)
 else:
  datemin = dateref[0]
  datemax = dateref[1] + pd.Timedelta(days=forecast_period)
  date_trunc = dateref[1]


 for i in range(nframes):
  idinclude = np.where((meta_df_in[i].values[:,0] > datemin) & (meta_df_in[i].values[:,0] < datemax))[0]
  df = meta_df_in[i].iloc[idinclude,:]
  #if normalise 1 then normalise
  if (normalise == 1):
   xn = df.values[:,1]
   xn0 = xn[np.nonzero(xn)]
   xnmean = np.mean(xn0)
   xnstd  = np.std(xn0)
   if (xnstd == 0):
    xnstd = 1
   else:
    xnstd = 1
    
   xnew = (xn - xnmean)/xnstd 
   df.values[:,1] = xnew  
   meta_df.append( df )
  
  

 #convert data frames to light curve format
 meta_lc = []
 meta_mod = []
 transform_mod = []
 fitstat_self = []
 fitstat_cross = []
 for i in range(nframes):
  x = c2l.convert_df_2_lightcurve(meta_df[i],reftime=datemin,noise = 0.01,normalise = 0)
  meta_lc.append(x)
 
  
  #model paramers
  if (i == 0):
   tlo = np.min(x[:,0])
   thi = np.max(x[:,0])
   dt  = x[1,0] - x[0,0]
   tgrid = np.arange(tlo,thi+dtgrid+forecast_period,dtgrid)
   flo = 1./(thi-tlo)
   fhi = 0.1/dtgrid
   ntgrid = np.shape(tgrid)[0]
   dategrid = [datemin + pd.Timedelta(days=i*dt) for i in range(ntgrid)]
   print('lower upper time grid limits',tlo,thi)
  
  
  
  
  
 #label each time series
 if (labels == []): 
  lab = ['time series '+np.str(i) for i in range(nframes)]
 else:
  lab = labels

  
  
  












  
  
  
 #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 #!!!!!!!!!!!!!!!!!! fit model to each time series !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 
 
 for idx_check in range(2):
  if ((idx_check == 1) and (accuracy_check == 0)):
   accuracy_envelope.append( np.nan )
   accuracy_variance.append( np.nan )
   continue
   

  for i in range(nframes):
   ntimes = np.shape(meta_lc[i][:,0])[0]
   if (idx_check == 1):
    idlast = ntimes - accuracy_check
   else:
    idlast = ntimes
   
   if (model == 'rw'):
    #x = mf18.fitrw([meta_lc[i][:idlast,0]],[meta_lc[i][:idlast,1]],[np.ones(ntimes)*np.std(meta_lc[i][:idlast,1])*0.1],floin=flo,
    #fhiin=fhi,plot_tit='show',dtresin=tgrid,nits = 10000,extra_f=[])
    #(tgrid,ygridop,f,ckout,skout)
    x = vrs.rw(meta_lc[i][:idlast,0],meta_lc[i][:idlast,1],si=0,tgi = tgrid,fbreak=-1)
    xmod = x[1][:,1]
    sigmod = (x[1][:,2] - x[1][:,0])/2
    fitstat = x[5]
   elif (model == 'poly'):
    
    x = vpf.fit(meta_lc[i][:idlast,0],meta_lc[i][:idlast,1],np.ones(ntimes)[:idlast],order=polyorder,xgrid=tgrid
    ,confidence=0.3173,nits=10000) 
    xmod = x[0]
    sigmod = (x[2] - x[1])/2
    fitstat = x[8]#rcoef
   
   elif (model == 'poly-rw'):
    xmod,sigmod,fitstat=vrwp.rwp(meta_lc[i][:idlast,0],meta_lc[i][:idlast,1],tgrid=tgrid,order=polyorder)
   
   elif (model == 'poly-custom'):
    if (custom_freqs == []):
     raise Exception('Must supply a numpy array of fit frequencies when chosing "poly_custom" model')
    
    xmod,sigmod,fitstat=vrwp.rwp(meta_lc[i][:idlast,0],meta_lc[i][:idlast,1],tgrid=tgrid,
    custom_freqs=custom_freqs,order=polyorder)
   #here we asses the model accuracy using cross validation testing on the last 
   #'accuracy_check' number of points
   

   
   
   if (idx_check == 0):
    meta_mod.append(np.array([dategrid,xmod,sigmod]).T) #meta_mod.append(np.array([tgrid,xmod,sigmod]).T) 
    fitstat_self.append(fitstat)
    

    
   else:
    tdat        = meta_lc[i][idlast:,0]
    xdat        = meta_lc[i][idlast:,1]
    ndat        = np.shape(xdat)[0]
    xmod_itp    = np.interp(tdat,tgrid,xmod)
    smod_itp    = np.interp(tdat,tgrid,sigmod)
    xmod_itp_hi = xmod_itp + smod_itp
    xmod_itp_lo = xmod_itp - smod_itp
    var = np.sum( (xmod_itp - xdat)**2 )/ndat
    rms = np.sqrt(var)
    mape = rms/np.mean(xdat)
    idgood = np.where( (xdat > xmod_itp_lo) & (xdat < xmod_itp_hi) )[0]
    ngood  = np.shape(idgood)[0]
    goodfrac = np.float(ngood)/accuracy_check
    accuracy_envelope.append( goodfrac )
    accuracy_variance.append( var )
    accuracy_mape.append( mape ) 
 
 
 
 
 
 
 
 
 
 
 
 #!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!
 #!!!!!!!!!perform operation on model to transform to flow time #!!!!!!!!#!!!!!!!!
 #!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!
 for i in range(nframes):
  if (i == 0 or transform_model == 'same'):
   response = meta_mod[i]
   response0 = meta_mod[0]
   t0lo = response0[0,0]
   t0hi = response0[-1,0]
   idinc = np.where((response[:,0] < t0hi) & (response[:,0] > t0lo))[0]
   sd0 = np.std(response0[idinc,1])
   sd  = np.std(response[idinc,1])
   mean0 = np.mean(response0[idinc,1])
   mean   = np.mean(response[idinc,1])
   rnew = (response[:,1] - mean)*sd0/sd + mean0
   sdnew = response[:,2]*sd/sd0
   transform_mod.append(np.array([dategrid,rnew,sdnew]).T)
  
   #plt.clf()
   #plt.plot(dategrid,transform_mod[0][:,1],color='k')
   #plt.plot(dategrid,rnew)
   #print('mean,sd,driver',mean0,sd0)
   #print('mean,sd,response',mean,sd)
   #print('mean,sd,response new',np.mean(rnew),np.std(rnew))
   #print('i',lab[i])
   #plt.show()
   #plt.clf()
   
   
  
  elif (transform_model == 'convolve'):
   lagcent,lagwide = convolve[i]
   lg = np.arange(ntgrid)*dt
   shape_k = np.int(2*(ntgrid-1) + 1)
   kernel = np.zeros(shape_k)
   laggrid = np.zeros(shape_k)
   k0 = np.int(np.floor(shape_k/2))
   laggrid = np.zeros(shape_k)
   laggrid[k0:]=lg
   laggrid[:k0]=lg[-1:0:-1]
   kernel= np.exp(-0.5*((laggrid-lagcent)/lagwide)**2)/(2*np.pi*lagwide)**2
   modnow = meta_mod[i][:,1]
   modmean = np.mean(modnow)
   modstd = np.std(modnow)
   response = apc.convolve(modnow,kernel)
   response_std = np.std(response)
   response_mean = np.mean(response)
   
   response = (response - response_mean)/response_std*modstd + modmean
   transform_mod.append(np.array([dategrid,response,sigmod]).T)#transform_mod.append(np.array([tgrid,response,sigmod]).T)
   #print('asdasdfafafa')
   #input()
   #for it in range(ntgrid):
   # print(dategrid[it],response[it],transform_mod[0][it,1])
   #print('transform check',i)
   #input()
 
  #asses the correlation between transformed model and driver
  mm = np.array(meta_mod[0][:,1],dtype='float')
  rm = np.array(transform_mod[i][:,1],dtype='float')
  rcoef = np.corrcoef(mm,rm)[0,1]
  fitstat_cross.append(rcoef)
 
 
 
 
 
 
 
 
 
 
 
 #!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!
 ##!!!!!!!!#!!!!!!!!#!!!!!!!! diagnostic plot #!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!
 #!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!
 if (figure_title != ''):
  ndown = nframes + 2
  nalong = 4
  plt.clf()
  fig = plt.figure()
  gs1 = gridspec.GridSpec(ndown,nalong)
  gs1.update(left=0.2,right=0.85,wspace=0.1,hspace=0.0,bottom=0.08,top=0.90)
   

   
  uncertanties_old = []
  uncertainties_new = []
  improvement_potential = np.zeros((ntgrid,nframes))
  for i in range(nframes):
   ax1 = plt.subplot(gs1[i,:])
   #ax1.scatter(meta_df[i].values[:,0],meta_lc[i][:,1],s=4,color=color[i])
   ax1.bar(meta_df[i].values[:,0],meta_lc[i][:,1],color=color[i],width = 25.0,fill=False)
   tmod    = tgrid#meta_mod[i][:,0]
   xmod    = np.array(meta_mod[i][:,1],dtype='float')
   xmodsig = np.array(meta_mod[i][:,2],dtype='float')
   xmodlo  = xmod - xmodsig
   xmodhi  = xmod + xmodsig
   
   if (i == 0):
    for i2 in range(1,nframes):
     xmod_trans = np.array(transform_mod[i2][:,1],dtype='float')
     ax1.plot(dategrid,xmod_trans,color=color[i2],label=lab[i2])
   
   ax1.plot(dategrid,xmod,label=None,color=color[i])
   ax1.plot(dategrid,xmod,label=None,color=color[i])
   
   
   
   #ax1.plot(dategrid,xmodlo,label=None,color=color[i])
   #ax1.plot(dategrid,xmodhi,label=None,color=color[i])
   #ax1.fill_between(dategrid,xmodlo,xmodhi,alpha=0.25,label=None,color=color[i])
   dates_dat       = meta_df[i].values[:,0]
   #values_dat      = meta_df[i].values[:,1] 
   #dates_start     = dates_dat[0]
   dates_end       = dates_dat[-1]
   df_now          = meta_mod[i]
   #try:
   idfc = np.where(df_now[:,0] > dates_end)[0][0]
   #except:
   #idfx = -1
    
   ax1.fill_between(dategrid[idfc:],xmodlo[idfc:],xmodhi[idfc:],alpha=0.25,label=None,color=color[i])
   
   ax1.set_xlabel('dates')
   #ax1.set_ylabel(lab[i],rotation = 0)
   ax1.annotate(lab[i],(-0.1,0.5),xycoords='axes fraction',horizontalalignment='right')
    
   #if i = 0 add on the forecasts using all the other models
   if (i == 0):
    idlo = np.where(tgrid > tgrid[-1]-forecast_period)[0][0]
    ntgrid = np.shape(tgrid)[0]
    points_lo=[]
    points_med=[]
    points_hi = []
    
    
    sig_old = []
    sig_new = []
    for i2 in range(0,ntgrid,1):
     points = np.array([transform_mod[i3][i2,1] for i3 in range(nframes)],dtype='float')
     sigma  = np.array([transform_mod[i3][i2,2] for i3 in range(nframes)],dtype='float')
     r      = np.array([ft for ft in fitstat_cross])
     r[r<0] = 0#dont use -ve ccf info (technically should but fix later)
     mean = np.sum(points*r/sigma**2)/np.sum(r/sigma**2)#ccf weighted mean for forecast
     sd  = np.sqrt(1/np.sum(r/sigma**2))
     sig_old.append(sigma)
     sig_new.append(sd)       
     pl,pm,ph = mean-sd,mean,mean+sd#np.percentile(points,[15,50,85])
     points_lo.append(pl)
     points_med.append(pm)
     points_hi.append(ph)
     #print(i2,i,improvement_potential[i2,i])
     if (i2 == 0):
      for ilc in range(nframes):
       improvement_potential[:,ilc] = 1. - np.sqrt( 1. / ( 1. + r[ilc]    ) )#np.sqrt( 1./sigma[0]**2 / ( 1./sigma[0]**2 + r[i]/sigma[i]**2    ) )
    points_lo  = np.array(points_lo)
    points_med = np.array(points_med)
    points_hi  = np.array(points_hi)
    #ax1.fill_between(tgrid[idlo:],points_lo[idlo:],points_hi[idlo:],alpha=1.0,color=color[0],label='multivariate forecast')
    #ax1.plot(tgrid[idlo:],points_lo[idlo:],alpha=1.0,color=color[0],ls='--',label='multivariate forecast')
    #ax1.plot(tgrid[idlo:],points_hi[idlo:],alpha=1.0,color=color[0],ls='--',label=None)
    ax1.bar(dategrid[idlo:],points_med[idlo:],alpha=0.3)
    
    #for it in range(10):
    # print(dategrid[idlo+it],points_med[idlo+it],
    
    ax1.legend()
    sigma_mean = np.mean(np.array(sig_old))
    newsigma_mean = np.mean(np.array(sig_new))
    #replace the error bars on the driving series with the updated errors from the combined forecast
    points_lo = np.array(points_lo)
    points_hi = np.array(points_hi)
    points_sd = (points_hi - points_lo) / 2
    transform_mod[0][idlo:,2] = points_sd[idlo:]
    meta_mod[0][idlo:,2] = points_sd[idlo:]
    
  
   #set plot limits
   ymin,ymax = np.min(meta_lc[i][:,1]),np.max(meta_lc[i][:,1])
   ax1.yaxis.tick_right()
   if (i == 0):
    ymin = min(ymin,np.min(points_lo[idlo:]))
    ymax = max(ymax,np.max(points_lo[idlo:]))
    ax1.set_title(figure_label)
   yrange    = ymax - ymin
   ax1.set_ylim([ymin - yrange/10,ymax + yrange/10])
   if (i < nframes - 1):
    ax1.set_xticks([])
    
   ax1.set_xlim([datemin,datemax])
    
  ##bar plot of self-fit statistics
  #ax1 = plt.subplot(gs1[ndown-1,:2])
  #xpos = np.arange(nframes)
  #[ax1.barh([xpos[i]],[fitstat_self[i]],label=None,color='k') for i in range(nframes)]
  #ax1.set_title('correlation coefficient') 
  #ax1.set_yticks(xpos) 
  #ax1.set_yticklabels(lab)
  #ax1.tick_params(axis='y',rotation=0)
  
  #bar plot of cross-fit statistics
  ax1 = plt.subplot(gs1[ndown-1,3:])
  xpos = np.arange(nframes)
  [ax1.barh([xpos[i]],[accuracy_mape[i]*100],label=None,color='k') for i in range(nframes)]
  ax1.set_title('Mape accuracy metric') 
  ax1.set_yticks(xpos) 
  ax1.set_yticklabels(lab)
  ax1.set_xticks(np.arange(0,110,50))
  ax1.tick_params(axis='y',rotation=0)
  ax1.set_xticklabels(ax1.get_xticklabels(),rotation=30)
  











 #!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!
 ##!!!!!output the model info into a dataframe to be converted to .csv spread sheet!!!!!#!!
 #!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!
 
 writer = pd.ExcelWriter(figure_title[:-4]+'.xlsx',engine='xlsxwriter',datetime_format='dd mm yyyy')
 uncertainty_mod = []
 for i in range(nframes):
  title = lab[i]
  #add in point-to-point scatter to uncertainty estimate
  dates_dat       = meta_df[i].values[:,0]
  values_dat      = meta_df[i].values[:,1] 
  dates_start     = dates_dat[0]
  dates_end       = dates_dat[-1]
  df_now          = meta_mod[i]
  values_mod      = df_now[:,1]
  mape_now        = np.zeros(ntgrid)
  mean_now        = np.mean(values_mod[values_mod != 0])
  output_now      = np.zeros((ntgrid,4),dtype='object')
  for it in range(ntgrid):
   dates_mod = df_now[it,0]
   sd_now    = df_now[it,2]
   sd_tot    = sd_now
   if ((dates_mod > dates_start) & (dates_mod < dates_end)):
    idx          = np.where(dates_dat < dates_mod)[0]
    ilo          = idx[-1]
    ihi          = ilo + 1
    scatter      = np.std(values_dat)
    sd_extra     = np.std(scatter)
    sd_tot       = np.sqrt(sd_now**2 + sd_extra**2)
    df_now[it,2] = sd_tot
   mape_now[it] = sd_tot/mean_now
  
  output_now[:,:3] = df_now
  output_now[:,3]  = mape_now*100  
  df_output    = pd.DataFrame(output_now,columns=['Dates','model','model_range (uncertainty)','Median Absolute Percentage Error (MAPE)'])
  idfc = np.where(df_now[:,0] > dates_end)[0]
  
  if (len(idfc) > 0):
   df_output[:idfc[0]].iloc[:,:3].to_excel(writer,sheet_name = title,)
   df_output[idfc[0]:].to_excel(writer,sheet_name = title+' forecasts')
   ut = tgrid[idfc[0]:]
   x = df_output[idfc[0]:].values[:,1:3]
   ux,us = x[:,0],x[:,1]
   
   xhere  = np.array([ut,ux,us]).T
   uncertainty_mod.append(xhere)
   
   #print(np.shape(xhere))
   #print(len(uncertainty_mod))
   #input()
 writer.save()






 #!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!
 ##!!!!!output forecast figure!!!!!#!!
 #!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!#!!!!!!!!
 nmonths = np.int(np.floor(forecast_period/30))
 nspace = 2

 xpos = np.arange(nmonths*(nframes+nspace))
 #fig = plt.figure()
 ysave = []
 for i in range(nframes):
  mm = np.array(uncertainty_mod[i])
  tm = mm[:,0]
  dtm = tm-tm[0]
  y = []
  values_dat      = meta_df[i].values[:,1] 
  mean = np.mean(values_dat[values_dat !=0])
  for im in range(nmonths):
   id = np.where(dtm > im*30)[0][0]
   pred = mm[id,1]
   sig = mm[id,2]
   pc = sig/mean
   y.append(pc*100)
  ysave.append(y)
 
 ax1 = plt.subplot(gs1[ndown-1,:2])
 [ax1.bar(xpos[i::nspace+nframes],ysave[i],label=lab[i],color=color[i]) for i in range(nframes)]
 lab_r = np.array(['']*len(xpos))
 idskip = np.int(np.floor(nframes/2))
 idr = np.arange(idskip,len(xpos),nframes+nspace)
 labs = [np.str(i) for i in range(nmonths)]
 for ir in range(nmonths):
  lab_r[idr[ir]] = labs[ir]
 

 ax1.set_xticks(xpos)
 ax1.set_xticklabels(lab_r)
 ax1.set_ylabel('MAPE')
 ax1.set_xlabel('forecast month')
 ax1.legend(fontsize='xx-small')
 ax1.tick_params(axis='x',which='both',length=0)
 
 if (figure_title !='show'):
  plt.savefig(figure_title) 
 else:
  plt.show() 
 
 


 return(fitstat_self,fitstat_cross,sigma_mean,newsigma_mean,
 accuracy_envelope,accuracy_variance,accuracy_mape,uncertainty_mod,improvement_potential) 
 
#return(parmout,covout,freq,tplot,xplot,xplotsd,xplotlo,xplothi,p0_out,w0,dw,sig2_prior,xplotsave)



##test make fake data
#import vortexa_makefake as mf
#tlo = 0
#thi = 100.0
#dt  = 1.0
#dfin = mf.makefake(tlo,thi,dt,shift = [10],noise=0.1,iseed=31234)
#
#
#a=mass_mod_test(dfin,dateref=-1,forecast_period = 50,figure_title='',model='poly',dtgrid=0.1,
#transform_model='convolve',convolve=[[0,0.1],[30,10.],[60,10]])






























#plot x and y using generalised poly fit
def reg_plot(xlist,ylist,lab = '',xlabel='',ylabel='',fig_title='reg_plot.pdf',nlag = 5):
 
 plt.clf()
 ny = len(ylist)
 rcoef = [0.0]*(ny)
 
 
 #try:
 ndown = 4
 nalong = ny 
 
 if (lab == ''):
  ax_title = ['']*ny
 else:
  ax_title = lab
 if (ylabel == ''):
  ylab = ['']*ny
 else:
  ylab = ylabel
 if (xlabel == ''):
  xlab = ['']*ny
 else:
  xlab = xlabel
  
 ngrid = 1000

 yglo = []
 yghi = []
 xgs = []
 ordersave = []
 lagsave = []
 ccfsave = []
 
 idx = 0
 for y in ylist:
  x = xlist[idx]
  idinclude = np.where((x > 0) & (y > 0))[0]
  
  
  
  
  #perform ccf 
  ccf = ss.correlate(x,y)
  nccf = np.shape(ccf)[0]
  neg = np.int(np.floor(nccf-1)/2)
  lag = np.zeros(nccf)
  idlag = np.arange(neg+1)
  lag[:neg] = idlag[-1:0:-1]
  lag[neg+1:] = idlag[1:]

  #identify lag that maximizes ccf only shift if abs(lag) < nlag maximum number of months
  lagmax = np.argmax(ccf)
  if (np.abs(lagmax) > nlag):
   lagmax = 0

  lagmax = 0
  kernel = np.zeros(nlag*2 + 1)
  kernel[nlag + lagmax] = 1

  ynew = apc.convolve(y, kernel)
  print(y)
  print(ynew)
  print()


  
  
  lagsave.append(lag)
  ccfsave.append(ccf)
  
  
  
  
  
  
  
  
  
  xmin,xmax = np.min(x),np.max(x)
  xrange = (xmax - xmin)/10
  xgrid = np.linspace(xmin-xrange,xmax+xrange,ngrid)
  xgs.append(xgrid)
  
  a = vpf.fit_search(x[idinclude],y[idinclude],maxorder=4,xgrid=xgrid)
  order_cisqred, order_aic, order_bic = a
  order = order_aic 
  ordersave.append(order)
  yg_med,yg_lo,yg_hi,cov,cisq,cisq_red,bic,aic,rmd = \
  vpf.fit(x[idinclude],y[idinclude],order=order,xgrid=xgrid,confidence=0.01,nits=20000,figure_title='')
  #correlation coefficient
  rcoef[idx] = np.corrcoef(x,y)[0,1] 
  yglo.append(yg_lo)
  yghi.append(yg_hi)
 #make figure
  idx = idx + 1
 
 gs1 = gridspec.GridSpec(ndown,nalong)
 gs1.update(left=0.1,right=0.98,wspace=0.4,hspace=0.0,bottom=0.2,top=0.9)
 for i in range(ny):
  xgrid = xgs[i]
  xmin,xmax = np.min(xgrid),np.max(xgrid)
  y = ylist[i]
  yg_lo = yglo[i]
  yg_hi = yghi[i]
  ymin,ymax = np.min(y),np.max(y)
  yrange = (ymax - ymin)/10
  ax1 = plt.subplot(gs1[0:3,i])
  ax1.scatter(x[idinclude],y[idinclude],color='k')
  ax1.plot(xgrid,yg_lo,color='b',label='fit')
  ax1.plot(xgrid,yg_hi,color='b',label=None)
  ax1.fill_between(xgrid,yg_lo,yg_hi,alpha=0.3,color='b',label=None)
  ax1.set_xlabel(xlab[i])
  ax1.set_ylabel(ylab[i])
  ax1.set_title(ax_title[i])
  ax1.set_xlim([xmin,xmax])
  ax1.set_ylim([ymin - yrange,ymax + yrange])
  ax1.annotate(r'$r_c = '+np.str(np.round(rcoef[i],2))+'$',(0.9,0.9),xycoords='axes fraction',horizontalalignment='right',color='b') 
  ax1.annotate('order = '+np.str(np.int(ordersave[i])),(0.9,0.8),xycoords='axes fraction',horizontalalignment='right',color='b') 
  
  
  
  #plot ccf figure only include first nlag time steps
  ax2 = plt.subplot(gs1[3,i])
  lag = lagsave[i]
  ccf = ccfsave[i]
  nccf = len(lag)
  nl0  = np.int(np.floor(nccf/2))
  ilo = nl0 - nlag
  ihi = nl0 + nlag
  ax2.bar(lag[ilo:ihi],ccf[ilo:ihi])
  ax2.set_xlabel('lag')
  ax2.set_ylabel(r'$r_c$')
  
  
  
  
  
  
  plt.savefig(fig_title)
   
 #except:
 # pass
 

 
 
 
 return(rcoef)



 
 
 
























import numpy as np
import matplotlib.pylab as plt
import glob
import pandas as pd
import datetime
import statsmodels.api as sm



def sarima(lc1,orderin=(1,1,0),seasonal_order = (0,1,0,120),trend='c',enforce_invertibility=False,
alpha_confidence=0.32,time_forecast=10.0,diagnostic_plot=''):

 time   = lc1[:,0]
 signal = lc1[:,1]
 dt = np.mean(time[1:]-time[:-1])
 nforecast = np.int(time_forecast/dt)
 ntime = np.shape(time)[0]
 tfclo = time[ntime-1]
 tfchi = time_forecast_lo + nforecast*dt
 idx_forecast_lo = ntime
 idx_forecast_hi = ntime + nforecast
 
 #fit the sarima model
 model   = sm.tsa.statespace.SARIMAX(endog=signal,
 order=order,
 seasonal_order=seasonal_order,
 trend='c',
 enforce_invertibility=False)
 results = model.fit()
 pred    = results.get_prediction(start = idx_forecast_lo, end= idx_forecast_hi )
 ps      = pred.summary_frame(alpha=alpha_confindence)
 pslo    = np.array(ps['mean_ci_lower'])
 pshi    = np.array(ps['mean_ci_upper'])
 npred   = np.shape(pslo)[0]
 


 #output the forecast time series
 ymean = np.array(ps['mean'])
 ylo  = np.array(ps['mean_ci_lower'])
 yhi  = np.array(ps['mean_ci_upper'])
 nym = np.shape(ymean)[0]
 xmod = np.linspace(tfclo,tfchi,nym)

  

 #plot the time series 9can also use dweek.plot() for simple option but less customisation
 if (diagnostic_plot != ''):
  x = time
  y = signal
  fig = plt.figure()
  ax1 = fig.add_subplot(111)
  ax1.plot(x,y,label='data')
  ax1.plot(xmod,ymean,label='forecast')
  ax1.set_title(lab)
  ax1.fill_between(xmod,ylo,yhi,alpha = 0.3,label='uncertainty')
  ax1.set_xlabel('Time')
  ax1.set_ylabel('light curve')
  
  plt.legend(fontsize='xx-small')
  if (disgnostic_plot != 'show'):
   plt.savefig(diagnostic_plot)
   plt.clf()
  else:
   plt.show()
   
   
 return(xmod,ymean,ylo,yhi)


import numpy as np




# go from difference of magnitudes to ratio of fluxes
# default option converts to mjy
def dmag2ratf(dmag,multiply=3631000.0):
    ratf=multiply*10**(-0.4*dmag)
    return(ratf)




## convert mags to fluxes (mJy)
def mabtonu(mab,sigmab = []):
    a = 3631000.0
    fnu=a*10**(-0.4*mab)
    if (sigmab == []):
     return(fnu)
    else:
     sigfnu = fnu*0.4*np.log(10.)*sigmab
     return(fnu,sigfnu)

#what if flux in mjy and error in mags  
def sigmab2sigfnu(fnumjy,sigmab):
 return(fnumjy*0.4*np.log(10)*sigmab)


# convert flam to fnu in jansky, flamin erg cm^-2 s^-1 A-1, wav in angsrom
def flam2fnu(flam,wavang):
 return(3.34e4 * wavang*wavang * flam)
 

#convert fnu to flam units as above jansky - erg cm^-2 s^-1 A-1
def fnu2flam(fnu,wavang):
 return(2.99401e-5 /(wavang*wavang) * fnu)
 

#convert erg s-1 cm -2 A-1 into watts/m2/hz
def flam2si(flam,wavang):
 a = 2.9792458e14
 return(flam/a/(wavang*wavang))


import numpy as np


#Kates outlier rejection criteria. Need to apply this to sdss light curves prior to merging
def myrejout_kate(tin,yin,sigyin,itmax = 5,tel='spec',epochdelete=[]):
 
 t = 1.*tin
 y = 1.*yin
 sigy = 1.*sigyin
 #deal with indicees to delete
 if (len(epochdelete) > 0):
  edrev = sorted(epochdelete,reverse=True)
  for idx in edrev:
   t = np.delete(t,idx)
   y = np.delete(y,idx)
   sigy = np.delete(sigy,idx)
 
 
 
 if (tel == 'phot'):
  ynew = 1.*y
  tnew = 1.*t
  signew = 1.*sigy
  nold = np.shape(ynew)[0]
  nnew = -666.
  it = 1
  
  
  while ((nnew != nold) and (it <= itmax)):
   
   nold = np.shape(ynew)[0]
   print nold
   
   #remove too-tiny fluxes
   idxinc_lo = np.where(np.abs(ynew) > 1.)[0]
   print len(idxinc_lo),'len(idxinc_lo)'
   #recalculate nmad, calculate median errors and stuff
   median = np.median(ynew)
   diff = np.abs(ynew - median)
   mad = np.median(diff)
   nmad = mad/0.6745
   med_err = np.median(signew)
   #print 'median,mad,nmad,med_err'
   #print median,mad,nmad,med_err
   #remove too-big fluxes
   idxinc_big = np.where(np.abs(ynew) < 5.*nmad)[0]
   print len(idxinc_big), 'len(idxinc_big)'
   
   #remove points with tiny errors
   idxinc_losig = np.where(signew > 0.2*med_err)[0]
   print len(idxinc_losig),'len(idxinc_losig)'
   
   #remove points with too big errors
   idxinc_hisig = np.where(signew < 5.*med_err)[0]
   print len(idxinc_hisig),'len(idxinc_hisig)'
   
   i1 = np.intersect1d(idxinc_lo,idxinc_big)
   i2 = np.intersect1d(i1,idxinc_losig)
   i3 = np.intersect1d(i2,idxinc_hisig)
   
   nnew = np.shape(i3)[0]
   ynew = y[i3]
   tnew = t[i3]
   signew = sigy[i3]
   it = it + 1
  
  
  #not sure how to do this exclude
  #remove entire light curve if too big offset dispersion
  #if (tel == 'bok'):
  # bound = 5000
  #elif (tel == 'cfht'):
  # bound = 600
  
  
  
  
 elif (tel == 'spec'):
  print np.shape(y)
  print np.shape(sigy)

  idxkeep = np.where((np.abs(y) > 0) & (sigy > 0))[0]
  tnew = t[idxkeep]
  ynew = y[idxkeep]
  signew = sigy[idxkeep]
 
 
 return(tnew,ynew,signew)
 
 
 
 
 
 
 
###test routine on kates old light curve bs new light curve to make sure you get the same points out
##rm_005 a good test
#
#file1 = '/Users/ds207/Documents/standrews/sta/lightcurves/sdss_all_lc_oct2017/updated_quasar_lcs/cream_save/rm_005/spec_i_rm005_i.dat'
##file1 = '/Users/ds207/Google Drive/rm_campaign_cream/katelc_group_1febmerge_0/rm005_gimerge/rm005_i_spec.dat'
#file2 = '/Users/ds207/Documents/standrews/sta/fort/fortcode/lucky_results/rm_005/spec_i_rm005_i.dat'
##file1 = '/Users/ds207/Google Drive/rm_campaign_cream/katelc_group_1febmerge_0/rm005_gimerge/rm009_i_cfht_a2i.dat'
##file2 = '/Users/ds207/Documents/standrews/sta/fort/fortcode/lucky_results/rm_005/cfht_i_rm009_i_rs.dat'

#file1 = '/Users/ds207/Documents/standrews/sta/lightcurves/sdss_all_lc_oct2017/updated_quasar_lcs/cfht_i/rm608_i_rs.dat'

#
#
#
#dat1 = np.loadtxt(file1)
#dat2 = np.loadtxt(file2)
#n1 = np.shape(dat1[:,0])[0]
#n2 = np.shape(dat2[:,0])[0]
#
#dat1new = myrejout_kate(dat1[:,0],dat1[:,1],dat1[:,2],tel='phot',epochdelete=[])
#dat2new = myrejout_kate(dat2[:,0],dat2[:,1],dat2[:,2],tel='spec',epochdelete=[18,13])
#
##dat1new = myrejout_kate(dat1[:,0],dat1[:,1],dat1[:,2],tel='phot',epochdelete=[13])
##dat2new = myrejout_kate(dat2[:,0],dat2[:,1],dat2[:,2],tel='phot',epochdelete=[18,13])
#
#n1new = np.shape(dat1new[0])[0]
#n2new = np.shape(dat2new[0])[0]
#
#
#
#print 'old lc before after',n1,n1new
#print 'newlc before after',n2,n2new
#
#
#
#



 
 
 


#!/usr/bin/python
#-*- coding: utf-8 -*-

import sys
import argparse
import numpy as np
from matplotlib import pyplot as plt
import scipy
import CCF_interp_py3 as myccf
from scipy import stats 

""" The following is a demonstration of how to use the CCF_interp.
It requires the two light curves (also provided) sample_lc1.dat
and sample_lc2.dat. It will output three data files containing the
CCF, CCCD, and CCPD, and one plot showing the light curves, CCF,
CCCD, and CCPD, all into the current directory. 
"""

########################################
###Read in two light curves
########################################
def kateccf(lc1,lc2,lag_range=[-100,100],interp = 2.0, nsim = 500, mcmode = 0, sigmode = 0.2, plotop = '', fileop = 0,output_file_folder = './',filecent='sample_centtab.dat',filepeak='sample_peaktab.dat',filesample='sample_peaktab.dat'):

 if type(lc1) is np.ndarray:
  mjd1,flux1,err1 = lc1[:,0], lc1[:,1], lc1[:,2]
 else:
  mjd1, flux1, err1 =  np.loadtxt(lc1, unpack = True, usecols = [0, 1, 2])
 
 if type(lc2) is np.ndarray:
  mjd2,flux2,err2 = lc2[:,0], lc2[:,1], lc2[:,2]
 else: 
  mjd2, flux2, err2 =  np.loadtxt(lc2, unpack = True, usecols = [0, 1, 2])
  
 #########################################
 ##Set Interpolation settings, user-specified
 #########################################
 #lag_range = [-100, 100]  #Time lag range to consider in the CCF (days). Must be small enough that there is some overlap between light curves at that shift (i.e., if the light curves span 80 days, these values must be less than 80 days)
 #interp = 2. #Interpolation time step (days). Must be less than the average cadence of the observations, but too small will introduce noise.
 #nsim = 500  #Number of Monte Carlo iterations for calculation of uncertainties
 #mcmode = 0  #Do both FR/RSS sampling (1 = RSS only, 2 = FR only) 
 #sigmode = 0.2  #Choose the threshold for considering a measurement "significant". sigmode = 0.2 will consider all CCFs with r_max <= 0.2 as "failed". See code for different sigmodes.
 
 ##########################################
 #Calculate lag with python CCF program
 ##########################################
 tlag_peak, status_peak, tlag_centroid, status_centroid, ccf_pack, max_rval, status_rval, pval = myccf.peakcent(mjd1, flux1, mjd2, flux2, lag_range[0], lag_range[1], interp)
 tlags_peak, tlags_centroid, nsuccess_peak, nfail_peak, nsuccess_centroid, nfail_centroid, max_rvals, nfail_rvals, pvals = myccf.xcor_mc(mjd1, flux1, abs(err1), mjd2, flux2, abs(err2), lag_range[0], lag_range[1], interp, nsim = nsim, mcmode=mcmode, sigmode = sigmode)
 
 lag = ccf_pack[1]
 r = ccf_pack[0]
 
 perclim = 84.1344746    
 
 ###Calculate the best peak and centroid and their uncertainties using the median of the
 ##distributions. 
 centau = stats.scoreatpercentile(tlags_centroid, 50)
 centau_uperr = (stats.scoreatpercentile(tlags_centroid, perclim))-centau
 centau_loerr = centau-(stats.scoreatpercentile(tlags_centroid, (100.-perclim)))
 print('Centroid, error: %10.3f  (+%10.3f -%10.3f)'%(centau, centau_loerr, centau_uperr))
 
 peaktau = stats.scoreatpercentile(tlags_peak, 50)
 peaktau_uperr = (stats.scoreatpercentile(tlags_peak, perclim))-centau
 peaktau_loerr = centau-(stats.scoreatpercentile(tlags_peak, (100.-perclim)))
 print('Peak, errors: %10.3f  (+%10.3f -%10.3f)'%(peaktau, peaktau_uperr, peaktau_loerr))
         
 
 ##########################################
 #Write results out to a file in case we want them later.
 ##########################################
 if (fileop == 1):
  centfile = open(output_file_folder+filecent, 'w')
  peakfile = open(output_file_folder+filepeak, 'w')
  ccf_file = open(output_file_folder+filesample, 'w')
  for m in xrange(0, np.size(tlags_centroid)):
      centfile.write('%5.5f    \n'%(tlags_centroid[m]))
  centfile.close()
  for m in xrange(0, np.size(tlags_peak)):
      peakfile.write('%5.5f    \n'%(tlags_peak[m]))
  peakfile.close()
  for m in xrange(0, np.size(lag)):
      ccf_file.write('%5.5f    %5.5f  \n'%(lag[m], r[m]))
  ccf_file.close() 
 
 
 ##########################################
 #Plot the Light curves, CCF, CCCD, and CCPD
 ##########################################
 if (plotop != ''):
  fig = plt.figure()
  fig.subplots_adjust(hspace=0.2, wspace = 0.1)
  
  #Plot lightcurves
  ax1 = fig.add_subplot(3, 1, 1)
  ax1.errorbar(mjd1, flux1, yerr = err1, marker = '.', linestyle = None, color = 'k', label = 'LC 1 (Continuum)')
  ax1_2 = fig.add_subplot(3, 1, 2, sharex = ax1)
  ax1_2.errorbar(mjd2, flux2, yerr = err2, marker = '.', linestyle = None, color = 'k', label = 'LC 2 (Emission Line)')
  
  ax1.text(0.025, 0.825, lc1, fontsize = 15, transform = ax1.transAxes)
  ax1_2.text(0.025, 0.825, lc2, fontsize = 15, transform = ax1_2.transAxes)
  ax1.set_ylabel('LC 1 Flux')
  ax1_2.set_ylabel('LC 2 Flux')
  ax1_2.set_xlabel('MJD')
  
  #Plot CCF Information
  xmin, xmax = -99, 99
  ax2 = fig.add_subplot(3, 3, 7)
  ax2.set_ylabel('CCF r')
  ax2.text(0.2, 0.85, 'CCF ', horizontalalignment = 'center', verticalalignment = 'center', transform = ax2.transAxes, fontsize = 16)
  ax2.set_xlim(xmin, xmax)
  ax2.set_ylim(-1.0, 1.0)
  ax2.plot(lag, r, color = 'k')
  
  ax3 = fig.add_subplot(3, 3, 8, sharex = ax2)
  #ax3.set_xlim(xmin, xmax)
  ax3.axes.get_yaxis().set_ticks([])
  ax3.set_xlabel('Centroid Lag: %5.1f (+%5.1f -%5.1f) days'%(centau, centau_uperr, centau_loerr), fontsize = 15) 
  ax3.text(0.2, 0.85, 'CCCD ', horizontalalignment = 'center', verticalalignment = 'center', transform = ax3.transAxes, fontsize = 16)
  n, bins, etc = ax3.hist(tlags_centroid, bins = 50, color = 'b')
  
  ax4 = fig.add_subplot(3, 3, 9, sharex = ax2)
  ax4.set_ylabel('N')
  ax4.yaxis.tick_right()
  ax4.yaxis.set_label_position('right') 
  #ax4.set_xlabel('Lag (days)')
  #ax4.set_xlim(xmin, xmax)
  ax4.text(0.2, 0.85, 'CCPD ', horizontalalignment = 'center', verticalalignment = 'center', transform = ax4.transAxes, fontsize = 16)
  ax4.hist(tlags_peak, bins = bins, color = 'b')
  
  plt.savefig(output_file_folder+plotop, orientation = 'landscape', bbox_inches = 'tight') 
  plt.close(plotop)
          
 return(tlags_centroid,tlags_peak,lag,r,centau,centau_uperr,centau_loerr,peaktau,peaktau_uperr,peaktau_loerr)
 
 


 
 


import numpy as np
import pandas as pd




def annotate_polygons(df,polgons,searchstring):

 #identify polygon id corresponding to search string
 ref_id = polygons[polygons['id'].str.contains(searchstring)]['id']

 #return all polygons that contain the reference id in their parent id's
 #i.e all the children of the searchstring
 npoly = len(polygons)
 idx_in = []
 for idx in range(npoly):
  parent_id = polygons['parent_id'][idx]
  if (ref_id in parent_id):
   idx_in.append(idx)
   

 polygons_in = polygons[idx_in]
 
 
 #return all the df columns that contain any of the polygon id's in polygons_in
 id_in = df['id'].isin(polygons_in)

 return(df[id_in])


import numpy as np
import vortexa_ccf_simple as mccf
import matplotlib.pylab as plt
import os
import matplotlib.gridspec as gridspec
import myconvolve as  mc3
import mylcgen as mlc
import myresample as mrs
import vortexa_mod_randomwalk as mf18
import pandas as pd
import datetime

#enter the time series signals as list of lists. Driver first [[time,values,noise],[time2,values2,noise2]...etc]
#titles for plots
def lag_rwmod(signals,titles=[],
 monte_carlo_iterations=10000,
 extra_frequencies = [],
 random_walk_gridres = 1.0,
 summary_plot='',
 nsim = 500,
 refdate=0,
 laglim=[]):
 
 
 time_model = []
 signal_model = []
 error_model  = []
 model_hi = []
 model_lo = []
 x_iterations = []
 
 gridres = random_walk_gridres
 nsignals = len(signals)
 
 
 #get the lag limits
 xmin = np.min(np.array( [np.min(signals[i][:,0]) for i in range(nsignals)] ))
 xmax = np.max(np.array( [np.max(signals[i][:,0]) for i in range(nsignals)] ))
 laghi = (xmax-xmin)/2
 laglo = -1*laghi
 
 if (laglim == []):
  lagrange = [laglo,laghi]
 else:
  lagrange = laglim

 

  
  
 if (refdate != 0):
  a = np.float(xmax-xmin)
  xmin = pd.datetime.date(refdate)
  xmax = pd.datetime.date(refdate) + pd.Timedelta(days=np.int(a))


 
 
 print('plotlim review',xmin,xmax,laglo,laghi)
 for snow in signals:
  
  tlo = np.min(snow[:,0])
  thi = np.max(snow[:,0])
  dt  = np.mean(snow[1:,0] - snow[:-1,0])
  
  tmod = np.arange(tlo,thi+gridres,gridres)
  nmod = np.shape(tmod)[0]
  x = mf18.fitrw([snow[:,0]],[snow[:,1]],[snow[:,2]],floin=1.0/(thi-tlo),fhiin=1.0/dt,ploton=0,plot_tit='fig_myrwfit',
  dtresin=tmod,nits = monte_carlo_iterations,extra_f=extra_frequencies)
 
 
  s_model = x[4]
  s_noise = x[5]
  npoints = np.shape(snow[:,0])[0]
  
  model_itp = np.interp(snow[:,0],tmod,s_model)
  #print(np.shape(snow[:,0]),np.shape(tmod),np.shape(s_model),np.shape(model_itp))
  refined_noise = np.sqrt(1./npoints* np.sum( (model_itp - snow[:,1])**2) )
  s_noise = np.ones(nmod)*refined_noise
  
  
  
  
  
  
  time_model.append(x[3])
  signal_model.append(s_model)
  error_model.append(s_noise)
  model_lo.append(s_model - s_noise)
  model_hi.append(s_model + s_noise)
  #model_lo.append(x[6])
  #model_hi.append(x[7])
  #error_model.append(x[5])
 
 centroids_model = []
 centroids_data = []
 ccf_save_save=[]
 lag_grid_save=[]
 smodel_drive = np.array([time_model[0],signal_model[0],error_model[0]]).T
 for i in range(nsignals):
  #compute ccf between the model driver and response
  lag_grid,ccf_save,tlags_peak,rlagspeak,tlags_data = mccf.ccf_frrss(signals[0],signals[i],dt=gridres,
  centroid_frac = 0.4,
  fraction_rss=0.8,nsim = nsim)
  centroids_data.append(tlags_data)
  
  
  smodel_now = np.array([time_model[i],signal_model[i],error_model[i]]).T
  
  
  #return(smodel_drive,smodel_now)
  lag_grid,ccf_save,tlags_peak,rlagspeak,tlags_model = mccf.ccf_frrss(smodel_drive,smodel_now,dt=gridres,fraction_rss=0.8,
  centroid_frac = 0.4,
  nsim = nsim,flux_randomisation=1)
  
  #print(i,nsignals,'here',summary_plot)
  #input()
  ##sreturn(smodel_drive,smodel_now)
  
  lag_grid_save.append(lag_grid)
  ccf_save_save.append(ccf_save)
  centroids_model.append(tlags_model)  
 
 

   
 #make figures showing each plot with ccf
 #compare WITH and WITHOUT Random Walk Prior
 if (summary_plot !=''):
  ndown = nsignals
  nalong = 4
  gs1 = gridspec.GridSpec(ndown,nalong)
  gs1.update(left=0.15,right=0.85,wspace=0.1,hspace=0.01,bottom=0.1,top=0.99)
  
  
  
   
 time_model_out = []
 for i in range(nsignals):
  ccf_save = ccf_save_save[i]
  lag_save = lag_grid_save[i]
  if (refdate == 0):
   time_dat = signals[i][:,0]
   time_mod = time_model[i]
  else:
   time_dat = np.array([pd.Timestamp(refdate) + pd.Timedelta(days=signals[i][i2,0]) for i2 in range(np.shape(signals[i][:,0])[0])])
   time_mod = np.array([pd.Timestamp(refdate) + pd.Timedelta(days=time_model[i][i2]) for i2 in range(np.shape(time_model[i])[0])])
  time_model_out.append(time_mod)
   
   
  if (summary_plot !=''): 
   ax1 = plt.subplot(gs1[i,1:])
   axccf = plt.subplot(gs1[i,0]) 
   ax1.errorbar(time_dat,signals[i][:,1],signals[i][:,2],marker='o',ls='',label='data',color='r')
   #ax1.errorbar(signals[i][:,0],signals[i][:,1],signals[i][:,2])
   ax1.plot(time_mod,signal_model[i],label='model',color='b')
   #ax1.annotate(titles[i],(0.1,0.9),textcoords='axes fraction',horizontalalignment='left')
   if (titles == []):
    t = 'signals '+np.str(i)
   else:
    t = titles[i]
    
   ax1.set_ylabel(t)
   #ax1.yaxis.tick_right()
   ax1.set_yticks([])
   ax1.yaxis.set_label_position('right')
   #ax1.fill_between(time_model[i],signal_model[i]-error_model[i],signal_model[i]+error_model[i],alpha=0.2)
   ax1.fill_between(time_mod,model_lo[i],model_hi[i],alpha=0.3,color='b',label='uncertainty region')
   axccf.hist(centroids_model[i],bins=20,alpha=0.4,label='with prior',color='b')
   
   axccf.hist(centroids_data[i],bins=20,alpha=0.4,label='without prior',color='r')
   axccf.set_xlim(lagrange) 
   axccf.set_yticks([])
   
   axccf_2 = axccf.twinx()
   
   
   #return(lag_grid,ccf_save)
   
   for i2 in range(len(ccf_save)):
    axccf_2.plot(lag_save,ccf_save[i2],color='k')

   #[axccf_2.plot(lag_grid,ccf_save[i],color='k') for i in range(len(ccf_save))]
   axccf_2.set_ylim([0,1.1])
   axccf_2.set_yticks([0.0,0.2,0.4,0.6,0.8,1.0])
   axccf_2.yaxis.tick_left()
   axccf_2.yaxis.set_label_position('left')
   axccf_2.set_ylabel('Correlation coefficient')
   #axccf_2.set_yticklabels(['',''])
   axccf_2.set_xlim(lagrange)
   #axccf_2.set_yticks([])
   #axccf_2.set_yticklabels([])
   ax1.set_xlim([xmin,xmax])
   if (i == 0):
    axccf.set_xticks([])
   
 if (summary_plot != ''): 
  axccf.legend() 
  ax1.legend()
  if (summary_plot=='show'):
   plt.show()
  else:
   plt.savefig(summary_plot)
  
  plt.clf()
   
 
 


 return(centroids_data,centroids_model,time_model_out,signal_model,error_model)
 
 
 
 
 
 
 
 
###test the lag_rwmod code
# 
#
#forecast = 180.0
#
#sddrive = 100.0
#meandrive = 0.0
#noise_drive = 0.8
#
#
#
#lagrange = [-100,100]
#sdecho = 1.0
#meanecho = 0.0
#noise_echo = 0.8 
#tlo = 0.0
#thi = 800.0
#dt  = 30.0
#gridres = 1.0
#lagres = 0.1
#nsim = 500
#
#
#
#
##impulse info
#lagcent = 50.0
#lagwide = 0.1
#lag = np.arange(0,lagrange[1]+lagres,lagres)
#response = np.exp(-0.5*((lag - lagcent)/lagwide)**2)
#
#
#
#
#
#
#
#
#
#datpre      = mlc.mylcgen(tlo=tlo-lagrange[1],thi=thi,dt=lagres,iseed=-1)#132423
#
#
#
#idlo  = np.where(datpre[:,0] > tlo)[0][0]
#npre     = np.shape(datpre[:,0])[0]
#
#
#datstd  = np.std(datpre[:,1])
#snow =  np.ones(npre)*noise_drive*datstd
#dat = mrs.myresample(dir='',fname=[''],dtave=dt,sampmin=20.,sampcode=3,datin=np.array((datpre[idlo:,0],datpre[idlo:,1],snow[idlo:])).T)
#ndat = np.shape(dat[:,0])[0]
#sig = dat[:,2]
#
#stdnow = np.std(dat[:,1])
#dat[:,1] = (dat[:,1] - np.mean(dat[:,1]))/stdnow*sddrive + meandrive
#dat[:,2] = dat[:,2]/stdnow*sddrive*noise_drive
#dat[:,1] = dat[:,1] + np.random.randn(ndat)*dat[:,2]
#
#
#
#x_drive = dat[:,1]
#errors_drive = dat[:,2]
#nx = np.shape(x_drive)[0]
#
#
#
#
#
#
#
#
#
##make a convolved version of the driver
#echo = mc3.mc3(datpre[:,0],datpre[:,1],lag,response)
#necho = np.shape(echo)[0]
#echostd = np.std(echo)
#errors_echo = np.ones(necho)*noise_echo*echostd
#dat_echo = mrs.myresample(dir='',fname=[''],dtave=dt,sampmin=20.,sampcode=3,datin=np.array((datpre[:,0],echo,errors_echo)).T)
#ndat = np.shape(dat_echo[:,0])[0]
#
#echosdnow = np.std(dat_echo[:,1])
#echomeannow = np.mean(dat_echo[:,1])
#
#dat_echo[:,1] = (dat_echo[:,1] - echomeannow)/echosdnow*sdecho + meanecho
#sig_echo = dat_echo[:,2]
#dat_echo[:,2] = sig_echo/echosdnow*sdecho*noise_echo
#dat_echo[:,1] = dat_echo[:,1] + np.random.randn(ndat)*sig_echo
#
#idlo_echo = np.where(dat_echo[:,0] > tlo)[0][0]
#
#
### fit modified random walk model
##nsnow = np.shape(snow)[0]
##d1 = np.zeros((nsnow,3))
##d1[:,:2] = datpre
##d1[:,2] = snow
##n_echo = np.shape(errors_echo)[0]
##d2 = np.zeros((n_echo,3))
##d2[:,0] = datpre[:,0]
##d2[:,1] = echo
##d2[:,2] = errors_echo
##
##
##id1lo = np.where((d1[:,0]>=tlo) & (d1[:,0]<=thi))[0]
##dat_new = 1.*dat
##dat_new[:,0] = dat_new[:,0] - 30.0
##
#
#
#
#
#
#
#denew = np.array(dat_echo[idlo_echo:,:])
##denew[:,0] = denew[:,0] + 50.0
#signals = [dat,denew]#[ dat, dat_new ]#[d1[id1lo,:],d2[id1lo,:]]
#titles  = ['arb','flows']
#
#a = lag_rwmod(signals,titles,summary_plot='show',monte_carlo_iterations=1)
#
#
#
##lag_grid,ccf_save,tlags_peak,rlagspeak,tlags_model = mccf.ccf_frrss(a[0],a[1])
#
##for each iteration of parameter values, compute the ratio of fourier transforms and 
##inverse to get the response function
##nits = np.shape(x_iterations[0][0,:])[0]
##response_save = []
##for it in range(nits):
## drive_ft = np.fft.fft(x_iterations[0][:,it])
## echo_ft  = np.fft.fft(x_iterations[1][:,it])
## response_ft = echo_ft/drive_ft
## response = np.fft.ifft(response_ft)
## response_save.append(response)
## 
##
##fig = plt.figure()
##ax1 = fig.add_subplot(111)
##[ax1.plot(time_model[0],rs) for rs in response_save[:100]]
###plt.savefig('fig_crazyidea_fftresponse.pdf')
###plt.clf() 
##plt.show()
#
#
#
#
# 
###save the sine and cosine value in  2d array
##cwt = np.ones((ndat,nw))
##swt = np.ones((ndat,nw))
##for iw in range(nw):
## cwt[:,iw] = np.cos(w[iw]*time)
## swt[:,iw] = np.sin(w[iw]*time)
##
##
##hnow_cc = np.tensordot(cwt.T/sig2,cwt,axes=1)#/sig2)
##hnow_sc = np.tensordot(swt.T/sig2,cwt,axes=1)#/sig2)
##
##hnow_cs = np.tensordot(cwt.T/sig2,swt,axes=1)#np.sum(cwt[:,iw]*swt[:,iwp]/sig2)
##hnow_ss = np.tensordot(swt.T/sig2,swt,axes=1)#np.sum(swt[:,iw]*swt[:,iwp]/sig2)
# 
# 
##lagcent = 34.0
##lagwide = 0.2
##lag = np.arange(0,lagrange[1]+lagres,lagres)
##response = np.exp(-0.5*((lag - lagcent)/lagwide)**2)
##
##ft = np.fft.fft(response)
##ift = np.fft.ifft(ft)
##plt.plot(np.real(ift),label='new')
##plt.plot(response,label='original')  
##plt.legend()
##plt.show()
#
#
#





#
#
#
#
#


#!!! ingest input data file, convert dmag to rat flux with mean level, also offset times by
#!!! desired amount

import numpy as np

def myfluxchange(fname, fmean, dtime, dat = 0):
 
 if (type(dat) == int):
  dat = np.loadtxt(fname)
 
 nx = np.shape(dat)
 ny = nx[1]
 nx = nx[0]
 datnew = np.zeros((nx,ny)) 
 print nx, ny
  
 datnew[:,0] = dat[:,0] + dtime		#times
 
 fnew = fmean*10**(-0.4*dat[:,1])	# data points
 
 sigfnew = np.abs (fmean * np.log(10.0) * 0.4*dat[:,2])
 
 
 datnew[:,1] = fnew
 
 datnew[:,2] = sigfnew
 
 for i in range(dat[:,0].shape[0]):
  print i, dat[i,0], dat[i,1], datnew[i,1], dat[i,2]
  raw_input()
 
 np.savetxt('mod_'+fname,dat)
 
 return()

 
 


import numpy as np
#import matplotlib as mpl
#mpl.use('Agg')
from astropy_stark.mycorner_jul6_16 import *
from astropy_stark.myedlum import *
from astropy_stark.mytemp0 import *
import matplotlib

#font = {'family' : 'normal',
#        'size'   : 20}
#
#matplotlib.rc('font', **font)


def cream_posterior(dirin,ibin = 2./3,
                    npoints = -1,
                    idT1=2,filepar='outputpars.dat',
                    extents_in=[],
                    header='',
                    idxres=[2,3,4],
                    special=[1,0,0],
                    true=['','',np.log10(0.75)],
                    fsave=''):
 #dat = np.loadtxt('/Users/ds207/Documents/standrews/sta/fort/fortcode/mcmcmultiresults/16feb_4151_alls/output_20170228_003/outputpars.dat')
 truei = list(true)
 #stack posterior probability distributions together
 icheck = isinstance(dirin,(list)) 
 ic = 0
 if (icheck == 1):
  for dir in dirin:
   try:
    d = np.loadtxt(dir+'/'+filepar,ndmin=2)
    print(dir+'/'+filepar,np.shape(d))
    ndown,nalong = np.shape(d)
    idlo = np.int(ibin*ndown) 
    if (ic == 0):
     dat = d[idlo:,:]
    else:
     dat = np.vstack((dat,d))
    ic = ic + 1 
   except:
    print('problem with output parameters in', dir+'/'+filepar)
  dir = dirin[0]
 else:
  dat = np.loadtxt(dirin+'/'+filepar,ndmin=2) 
  dir = dirin

 
 
 if (filepar == 'outputpars.dat'):
  datnorm = 1
 #first off do the accretion disc parameters then try the top hat centroids 
 #try:
 # 
 # datnorm = 1
 # idxres  = [2,3,4]
 # special = [1,0,0]#is special = 2 take arc cos of data before potting
  
 #except:
 # print 'no outputpars.dat file must be in tfx mode',dir+'/outputpars.dat'
 # dat = np.loadtxt(dir+'/outputpars_tfxdisk.dat')
 # datnorm = 0
 # idxres = [0,1,2,3,4]
 # special = [0,0,1,1,0]
 
 
 try:
  ed   = np.loadtxt(dir+'/plots/cream_miscpar.dat')
  embh = ed[0]
  eta  = ed[1]
 except:
  embh = 1.e7
  eta  = 0.1
 
 
 
 
 
 
 #find if mmdot is varied
 if (datnorm == 1):
  try:
   imdot = np.where(np.array(idxres) == 2)[0][0]
   if (special[imdot] == 1):
    logmmdot = 1
   else:
    logmmdot = 0
  except:
   logmmdot = 0
   imdot = -1
  
  
  
  
  
  #if T1 = 1 express temperatures as T1 here
  if (idT1 > 0):
   idslope = 4
   emlog = np.log10(embh)
   dotmmlog = emlog+np.log10(dat[:,idT1])
   alphain  = dat[:,idslope]
   temp = temp0(dotmmlog, emlog, sigdotmmlog =0, sigemlog = 0.0, alpha_in = 0.75, sig_alphain=0.0,eta=0.1, alb=0, hxrs = 3,r0ld = 1)
   temp_info = temp[0]
   if (truei[0] != ''):
    truei[0] = np.log10(temp0(emlog+np.log10(truei[0]), emlog, sigdotmmlog =0, sigemlog = 0.0, alpha_in = 0.75, sig_alphain=0.0,eta=0.1, alb=0, hxrs = 3,r0ld = 1)[0])
 
 
 
 nver = np.shape(dat)[0]
 
 
 idxlo = int(np.floor(ibin*nver))
 dat = dat[:,idxres]
 nver  = np.shape(dat[:,0])[0]
 nhor = np.shape(dat)[1]
 
 
 #calculate edingon ratios
 #embh = 3.e7
 #eta = 0.1
 
 
 
 
 
 

 
 
 ivar = [1]*nhor
 for i in range(nhor):
  idspec = special[i]
  if (idspec == 2):
   dat[:,i] = np.arccos(dat[:,i])*180/np.pi
  if (idspec == 1):
   dat[:,i] = np.log10(dat[:,i])
  
   
   
  sd = np.std(dat[:50,i])
  if (sd < 1.e-10):
   ivar[i]=0
 
 ivar = np.array(ivar)
 idxvar = list(np.where(ivar > 0)[0])
 #idxvar = idxvar[:-1]

 
 

 dat = dat[:,idxvar]

 
 #function to cast number in standard form
 def sci_notation(number, sig_fig=2):
   ret_string = "{0:.{1:d}e}".format(number, sig_fig)
   a,b = ret_string.split("e")
   b = int(b) #removed leading "+" and strips leading zeros too.
   return "$"+a + " \\times 10^" + str(b) +"$"
 
 
 
 if (((embh > 0) or (imdot != -1)) and (datnorm == 1)):
  if (logmmdot == 1):
   dotm = 10**dat[:,0]
  else:
   dotm = dat[:,0]
  edd_rat = edd(embh,dotm,eta)[2]
  idxsort     = np.argsort(edd_rat)
  ns          = np.shape(idxsort)[0]
  ers         = edd_rat[idxsort]
  idlo        = int(max(0,np.floor(0.17*ns)))
  idhi        = int(min(np.ceil((1.-0.17)*ns),ns-1))
  ermed       = ers[ns/2]
  erlo        = ers[idlo]
  erhi        = ers[idhi]
  siglo       = ermed - erlo
  sighi       = erhi  - ermed
  aner        = ', $L/L_{edd}='+np.str(np.round(ermed,2))+'_{-'+np.str(np.round(siglo,2))+'}^{+'+np.str(np.round(sighi,2))+'}$ for $M_\\mathrm{BH}=$'+sci_notation(embh)+'$M_\odot$'
 else:
  aner = ''
 
 if (npoints == -1):
  iskip = 1
 else:
  iskip = nver/npoints
 
 
 
 #truths = [p0,w0,alpha,beta]
 #annotate = [r'$\dot{M}/M_\odot$yr$^-1$',r'$\cos \left( i \right)$',r'$\alpha$']
 
 
 
 costick = np.cos(np.pi/180*np.array([0,5,10,15,20,25,30,35,40,45,50,55,60,65,70,75,80,85,90])) 
 #coslab = [np.str(np.arccos(costick[i])*180/np.pi) for i in range(len(costick))]
 coslab = ['0','','','','','','','','40','','','','60','','','','80','','']
 #coslabblank = ['' for i in range(len(costick))]
 
 if (datnorm == 1):
  if (extents_in == []):
   extents=[0.9999,(0.,1.),0.9999]
  else:
   extents=extents_in
  
  annotate = [r'$\log ( T_1 $/K$  )$',r'$i$',r'$\log \alpha$']
  ann      = ['','','']#[aner,'','']
  smpi     = [r'$\log ( T_1 $/K$  )$',r'cos $i$',r'$\log \alpha$']
  specax = [[],costick,[]]
  specaxlab = [[],coslab,[]]
  if (idT1 > 0):
   annotate[0] = r'$\log T_1$ (K)'
   smpi[0] = r'$\log T_1$ (K)'
 
 else:
  if (extents_in == []):
   extents=[0.9999,(0.,1.),0.9999]
  else:
   extents=extents_in

  annotate = [r'$\log \alpha_v$',r'$\log \alpha_i$',r'$T_{1v}$',r'$T_{1i}$',r'$i$']
  ann      = ['']*5
  smpi     = [r'$\log \alpha_v$',r'$\log \alpha_i$',r'$T_{1v}$',r'$T_{1i}$',r'$i$']
  specax = [[],[],[],[],costick]
  specaxlab = [[],[],[],[],coslab]
  
 
 #print 'idx var...',idxvar
 a = [annotate[i] for i in idxvar]
 b = [smpi[i] for i in idxvar]
 c = [extents[i] for i in idxvar]
 d = [ann[i] for i in idxvar]
 specax = [specax[i] for i in idxvar]
 specaxlab = [specaxlab[i] for i in idxvar]
 truth_in = [truei[i] for i in idxvar]
 
 #extents=[(0.,1.5),(0.,90.)]
 ann_in   = [annotate[i] for i in idxvar]
 extra_an = [' ' for i in idxvar]#[ann[i] for i in idxvar]
 #a = mc.corner_1(dat,plot_contours=0,sigconts=[100.-68,100.-95,100-99.7],skip = iskip,plot_datapoints=True,
 #sigmedann_pre_in=b,labels=a,extents=[0.99,(0.,90.)],figname = 'parmparmplot.pdf')
 
 
 
 datcorner = np.array(dat)
 if ((idT1> 0) and (datnorm == 1)):
  datcorner[idxlo:,0] = np.log10(temp_info[idxlo:])
 
 #tr slope should be plotted in log
 for idnow in range(len(b)):
  if ('alpha' in b[idnow]):
   datcorner[idxlo:,idnow] = np.log10(dat[idxlo:,idnow])
  
  
 if (fsave==''):
  figname = 'posterior.pdf'
 else:
  figname = fsave
 
 print('making posterio plots...cream_posterior', truth_in)
 a = corner_1(datcorner[idxlo:,:],plot_contours=0,sigconts=[100.-68],skip = iskip,plot_datapoints=True,sigmedann_pre_in=b,sigmedann_post_in = d, 
 labels=a,extents=c,annotate = extra_an, sigmedann=1, 
 xtxtcoord = 1.25,ytxtcoord=1.05,lbm_in = 0.65,ltr_in = 0.4,xlab_coord=-0.3,ylab_coord=-0.3,
 specax=specax,specaxlab=specaxlab,truths=truth_in,title=header, 
 truth_color='r', figname = fsave)
 #except:
 # print 'cannot make continuum mdot vs inc posterior plot (maybe no continuum light curves)'
 # pass
 
 
 #try to make line plots of all the varied parameters
 try:
  nvar = len(idxvar)
  fig = plt.figure()
  for i in range(nvar):
   ax1 = fig.add_subplot(nvar,1,i+1)
   ax1.plot(dat[:,i])
   ax1.set_xlabel('Iteration')
   ax1.set_ylabel(annotate[i])
  plt.savefig('fig_cream_lines.pdf')
 except:
  pass
 
 #now do for the top hat parameters
 try:
  npoints = 200
  dat = np.loadtxt('../outputpars_th.dat') 
  nlc = np.shape(dat[0,:])[0]/2
  anthcen = ['lc (cent) '+str(i) for i in range(nlc)]
  anthwid = ['lc (fwhm) '+str(i) for i in range(nlc)]
  
  ann = anthcen + anthwid
  
  
  
  #check which points to plot
  npar = 2*nlc
  idxin = [i for i in range(npar) if np.shape(np.unique(dat[:,i]))[0] > 1]
  dat = dat[:,idxin]
  ann = [ann[i] for i in range(npar) if i in idxin ]
  
  nver = np.shape(dat[:,0])[0]
  iskip = max(1,nver/npoints)
   
  a = corner_1(dat[idxlo:,:],plot_contours=0,sigconts=[100.-68],skip = iskip,plot_datapoints=True,
  labels=ann, title=title, xtxtcoord=1.45,ytxtcoord=0.95,lbm_in = 0.65,ltr_in = 0.4,
  xlab_coord=-1.9,ylab_coord=-1.9,figname = 'posterior_th.pdf')
 except:
  print('cannot make top hat posterior plot (maybe no BLR top hat light curves)')
  pass
  
  
 return()



import numpy as np
import pandas as pd





#input list of pandas data frames (timestamp, quantity) with different timestamps
#rebin onto single axis and convert to numpy array

#merge dates of all other frames onto first (if no date entry matching up with first 
#date frame then input nan

def df2np(df_list,datemin=-1):
 nlist = len(df_list)
 df_combine = df_list[0]
 df_0 = df_list[0]
 n0 = len(df_0)
 output_array = np.zeros((n0,nlist))
 output_array[:,0] = df_0.values[:,1]
 
 for i in range(nlist):
  df_now = df_list[i]
  columns = df_now.columns
  cdate = columns[0]
  columns0 = df_combine.columns
  cdate0 = columns0[0]  
  df_0[cdate0] = pd.to_datetime(df_0[cdate0])
  df_now[cdate] = pd.to_datetime(df_now[cdate])
  
  #print(cdate,cdate0)
  #print(df_now)
  
  df_new = pd.merge(df_0, df_now, left_on = cdate0, right_on = cdate,how='left')
  #c_new = df_new.columns
  vals_new = df_new.values[:,-1]
  #df_combine = pd.merge(df_combine, df_new, left_on = cdate0, right_on = cdate,how='left')
  output_array[:,i] = vals_new

 
 dates = df_0.values[:,0]#df_combine.values[:,0] 
 output_data = output_array#df_combine.values[:,1:]
 
 #print(dates)
 
 #print(df_combine.columns)
 #print(output_data[:10,:])
 #print(output_data.shape)
 sums = np.sum(output_data,axis=1)
 idgood = np.where(sums == sums)[0]
 dates = dates[idgood]
 output_data = output_data[idgood,:]
 
 return(dates,output_data)


# routine to calculate the nth cenral moment of an input distribution
# Bug fixed 15/10/14 incorrect calculation of skew and kurt fixed now
import numpy as np
import pylab as plt

def mysig2(xdist,ydist,x0,area,a):
 
 sum = np.sum(ydist*a*a)
 return(sum/area)


def mystatmoments(xdist,ydist,x0,nth):
 
 a = xdist - x0
 ndist = xdist.shape[0]
 dx = (xdist[-1] - xdist[0])/(ndist-1)
 area = np.sum(ydist)
 
 sig2 = mysig2(xdist,ydist,x0,area,a)
 if (nth == 2): 
  answer = sig2
 elif (nth == 3):
  aa = a*a
  sum = np.sum(ydist*aa*a)
  answer = sum/area/(sig2**1.5)  #before 15/10/14 just divided by sig2 and not sig^n
 elif (nth ==4):
  aa= a*a
  sum = np.sum(ydist*aa*aa)
  answer = sum/area/(sig2*sig2)
 
 return(answer)
 
 
##!!!!!! test here
#x = np.arange(0,100,0.00001)
#
#x0 = 15.0
#sig = 4.0
#
#a = 10/(np.sqrt(2*np.pi*sig*sig))
#
#y = a * np.exp(-0.5*(x-x0)**2/(sig*sig))
##
#
#opsig = mystatmoments(x,y,x0,2)
#opskew = mystatmoments(x,y,x0,3)
#opkurt = mystatmoments(x,y,x0,4)
##
#print opsig, opskew, opkurt, x.shape[0]



 


#code to compare the model and lag estiamtes for cream fits with different
#error bar treatments for a few example targets
import numpy as np
import matplotlib.pylab as plt
import matplotlib.gridspec as gridspec
import glob
import astropy_stark.cream_posterior as cpos
idnow = 0

class plot_library:

 def __init__(self,directory):
  self.dnow = directory
  self.title = ''
  self.idburnin = 2. / 3
  self.justth = 0
  self.justcont = 0
  self.plotinfo = 1
  self.plottrace = 0
  self.plots_per_page = 5
  self.xlclab = 'Time (HJD - 50,000)'
  self.xtflab = 'lag (days)'
  self.forcelab = []
  self.forcelag = []
  self.sameplotdrive = 1
  self.extents = []
  self.justnewsig = 0
  self.taumeanplot = 1
  self.tau90plot = 0
  self.postplot = 1
  self.header = ''
  self.tauplot0 = 0
  self.gplot = 1
  self.true = ['', '', np.log10(0.75)]

  self.load_info()


 def load_info(self):
  idburnin = self.idburnin
  title = self.title
  header = self.header
  taumean = 1
  idcount = 1
  dirres = []
  tit = []
  head = []
  dnow = self.dnow
  if 'output_20' in dnow:
   dirres.append(dnow)
  else:
   a = sorted(glob.glob(dnow + '/output_20*'))[-1]
   dirres.append(a)
  if (title == ''):
   tit.append('lcplot_' + np.str(idcount))
  elif isinstance(title, str):
   tit.append(title + '_' + np.str(idcount))
  else:
   tit = list(title)
  head.append(header)
  self.head = head

  '''
  load data from relevant files
  '''
  print('cream_lcplot plotting results from...', dnow)
  fileth = '/outputpars_th.dat'
  filetf = '/plots/modeltf.dat'
  filetfsig = '/plots/modeltf_sig.dat'
  filemod = '/plots/modellc.dat'
  filemodsig = '/plots/modellc_sig.dat'
  filevarexp = '/outputpars_varexpand.dat'
  filesigexp = '/outputpars2.dat'
  filedrive = '/plots/modeldrive.dat'
  filebof = '/testbofs.dat'
  filepspec = '/cream_furparms.dat'

  ndres = len(dirres)
  for idnow in range(ndres):

   # put this in a loop to make plots for all tested targets
   dnow = dirres[idnow]

   # read cream names file
   f = open(dnow + '/../creamnames.dat')
   filedat = f.readlines()
   filedat = [i.strip() for i in filedat]  # remove \n
   wav = [i.split(' ')[-1] for i in filedat]
   wav = [float(x) for x in wav]
   filedat = [i.split(' ')[0] for i in filedat]
   filedat = [i[1:-1] for i in filedat]
   f.close()
   nwav = len(wav)

   # !!!!!!!!!!!!!!!!!!!!!! load the model (all wavelengths in one file) !!!!!!!!!
   try:
    mod = np.loadtxt(dnow + '/' + filemod)
   except:
    mod = np.loadtxt(dnow + filemod)

   idxinc = np.where(mod[:, 1] != 0)[0]
   tlo, thi = mod[idxinc[0], 0], mod[idxinc[-1], 0]
   mod = mod[idxinc, :]
   sigmod = np.loadtxt(dnow + '/' + filemodsig)
   tmod = mod[:, 0]
   mod = mod[:, 1:]
   sigmod = sigmod[idxinc, 1:]
   # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   # !!!!!!!!!!!!!!!!!!!!! load the response function !!!!!!!!!!!!!!!!!!!!!!!!!!!
   modtf = np.loadtxt(dnow + '/' + filetf)
   sigtf = np.loadtxt(dnow + '/' + filetfsig)
   tau = modtf[:, 0]
   modtf = modtf[:, 1:]
   sigtf = sigtf[:, 1:]

   # try to load top hat parameters, replace the disk response if non zero
   modth = np.loadtxt(dnow + '/' + fileth)[:, :nwav]
   nits = np.shape(modth[:, 0])[0]
   modth = modth[int(idburnin * nits):, :]
   thstats = np.percentile(modth, [15.865, 50, 84.135], axis=0)
   thmed = thstats[1, :]
   idxth = np.where(thmed != 0)[0]
   # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   # !!!!!!!!! load the error bar modification parameters calculate the median parameter and apply it
   try:
    sigexp = np.loadtxt(dnow + '/' + filesigexp)[:, 2 * nwav:]
    nits = np.shape(sigexp[:, 0])[0]
    sigexp = sigexp[int(idburnin * nits):, :]
    semed = np.median(sigexp, axis=0)
   except:
    print('no error bar f factors')
    semed = np.ones(nwav)

   try:
    varexp = np.loadtxt(dnow + '/' + filevarexp)
    nits = np.shape(varexp[:, 0])[0]
    varexp = varexp[int(idburnin * nits):, :]
    vamed = np.median(varexp, axis=0)
   except:
    print('no var expand parameters')
    vamed = np.zeros(nwav)

   try:
    # data required for outlier rejection
    fileoutrej = [dnow + '/plots/data_echo_dat_' + filedat[ilc] for ilc in
                  range(len(wav))]  # sorted(glob.glob(dnow+'/plots/data_echo_dat_*'))
    drej = [np.loadtxt(fileoutrej[idx]) for idx in range(nwav)]
   except:
    print('no outlier rejection found in...', dnow + '/plots/data_echo_dat_*')
    drej = [np.zeros((1, 4)) for idx in range(nwav)]
  self.idxinc = idxinc
  self.semed = semed
  self.vamed = vamed
  self.tau = tau
  self.modtf = modtf
  self.drej = drej
  self.idxth = idxth
  self.sigtf = sigtf
  self.tmod = tmod
  self.mod = mod
  self.tlo = tlo
  self.thi = thi
  self.sigmod = sigmod
  self.taumean = taumean
  self.modth = modth
  self.nwav = nwav
  self.tit = tit




 def plot_trace(self):
  plt.close()
  head = self.head
  nwav = self.nwav
  '''
  :return:
  '''
  dnow = self.dnow
  filetrace = ['/outputpars.dat','/outputpars2.dat','/outputpars_th.dat','/outputpars_varexpand.dat']
  idtrace = [[2,3,4],list(np.arange(3*nwav)),list(np.arange(2*nwav)),list(np.arange(nwav))]
  tit_trace = [[r'$\log \dot{M}$',r'$\cos i$',r'$\alpha$'],
  ['stretch '+np.str(idx) for idx in range(nwav)] + ['offset '+np.str(idx) for idx in range(nwav)] + ['sig f '+np.str(idx) for idx in range(nwav)],
  [r'$\tau_\mathrm{cent}$ '+np.str(idx) for idx in range(nwav)] +  [r'$\tau_\mathrm{width}$ '+np.str(idx) for idx in range(nwav)],
  ['sig_V '+np.str(idx) for idx in range(nwav)]
  ]

  nft = len(filetrace)
  idtot  = 0
  ptraceop = []
  ttraceop = []
  for idx in range(nft):
   npar = len(idtrace[idx])
   try:
    dtrace = np.loadtxt(dnow+filetrace[idx])[:,idtrace[idx]]
    vartrace = np.std(dtrace,axis=0)
    idinc = np.where(vartrace > 0)[0]
    d_trace = [dtrace[:,idx2] for idx2 in idinc]
    t_trace = [tit_trace[idx][id2] for id2 in range(npar) if id2 in idinc]
    ptraceop.append(d_trace)


    ttraceop.append(t_trace)
   except:
    print('cannot find file for trace plots in...')
    print(dnow+filetrace[idx])

  ptrace_plot = [j for i in ptraceop for j in i]
  ttrace_plot = [j for i in ttraceop for j in i]

  ntrace = len(ptrace_plot)
  nalong = min(4,ntrace)


  ndown = np.int(np.ceil(1.*ntrace/nalong))
  gs1 = gridspec.GridSpec(ndown, nalong)
  gs1.update(left=0.1, right=0.9, wspace=0.05,hspace = 0.0,bottom=0.1,top=0.99)

  iynow = -1
  for idx in range(ntrace):
   ixnow = np.mod(idx,nalong)
   if (ixnow == 0):
    iynow = iynow + 1
   ax1 = plt.subplot(gs1[iynow, ixnow])
   ax1.plot(ptrace_plot[idx])
   ax1.set_ylabel(ttrace_plot[idx])
  ax1.set_title(head[idnow])

  return(ax1)






 def plot_driver(self):
  plt.close()
  filedrive = '/plots/modeldrive.dat'
  filebof = '/testbofs.dat'
  filepspec = '/cream_furparms.dat'
  dnow = self.dnow
  boffile = np.loadtxt(dnow + filebof)
  cisq = boffile[:,0]
  idxinc = self.idxinc
  head = self.head

  if (self.plotinfo == 1):
   boffur = boffile[:,1]
   boftot = cisq + boffur

   pspecfile = np.loadtxt(dnow + filepspec,skiprows=1)
   freq = pspecfile[1:,0]/2/np.pi
   pspec = pspecfile[1:,[1,2]]

   datdrive = np.loadtxt(dnow + filedrive)



   ndown = 2
   nalong = 2
   gs1 = gridspec.GridSpec(ndown, nalong)
   gs1.update(left=0.1, right=0.9, wspace=0.3,hspace = 0.3,bottom=0.1,top=0.99)

   ax1 = plt.subplot(gs1[0, :])
   ax1.plot(datdrive[idxinc,0], datdrive[idxinc,1],color='k',linewidth=0.5)
   ax1.fill_between(datdrive[idxinc,0],datdrive[idxinc,1]-datdrive[idxinc,2],datdrive[idxinc,1]+datdrive[idxinc,2],alpha = 0.3,color='k')
   ax1.plot(datdrive[idxinc,0], datdrive[idxinc,1]-datdrive[idxinc,2],color='k')
   ax1.plot(datdrive[idxinc,0], datdrive[idxinc,1]+datdrive[idxinc,2],color='k')
   ax1.set_xlabel('time (days)')
   ax1.set_ylabel('X(t) (arbitrary units)')

   ax1 = plt.subplot(gs1[1,0])
   ax1.plot(boftot,label=r'$\chi^2+\mathrm{BOF}_\mathrm{fur}$' )
   ax1.plot(cisq,label=r'$\chi^2$')
   ax1.plot(boffur,label=r'$\mathrm{BOF}_\mathrm{fur}$')
   ax1.set_xlabel('Iteration')
   ymin = np.min(boffur)
   ymax = np.mean(boftot) + np.std(boftot)*2#ymin + np.max(
   ylim = [ymin,ymax]
   ax1.set_ylim(ylim)
   ax1.set_ylabel('BOF')
   plt.legend(fontsize='small')
   ax1 = plt.subplot(gs1[1,1])
   ps = pspec[:,0]**2 + pspec[:,1]**2
   ax1.scatter(freq,ps)
   try:
    sigpspec = pspecfile[1:,[3,4]]
    ps_sd = 2* np.sqrt( (pspec[:,0]*sigpspec[:,0])**2 + (sigpspec[:,1]*sigpspec[:,1])**2 )
    ax1.errorbar(freq,ps,ps_sd)
    #ax1.fill_between(freq,ps,ps-ps_sd,ps+ps_sd)
   except:
    print('cannot find uncertanties for power spectrum plot')

   xlim = [freq[0],freq[-1]]
   ax1.set_xlim(xlim)
   ax1.set_xscale('log')
   ax1.set_yscale('log')
   ax1.set_xlabel('frequency (cycles/day)')
   ax1.set_ylabel('P(f)')
   ax1.set_title(head[idnow])

   return(ax1)
   #plt.savefig('fitinfo_'+np.str(tit[idnow])+'.pdf')





 def plot_lightcurves(self):

  plt.close()
  axtf = None
  dnow = self.dnow
  justcont = self.justcont
  justth = self.justth
  plots_per_page = self.plots_per_page
  sameplotdrive = self.sameplotdrive
  semed = self.semed
  vamed = self.vamed
  justnewsig = self.justnewsig
  taumeanplot = self.taumeanplot
  tau90plot = self.tau90plot
  tauplot0 = self.tauplot0
  forcelab = self.forcelab
  forcelag = self.forcelag
  idxinc = self.idxinc
  idxth = self.idxth
  tlo, thi = self.tlo, self.thi
  mod = self.mod
  sigmod = self.sigmod
  tit = self.tit

  fig_objects = []



  # read cream names file
  f = open(dnow + '/../creamnames.dat')
  filedat = f.readlines()
  filedat = [i.strip() for i in filedat]  # remove \n
  wav = [i.split(' ')[-1] for i in filedat]
  wav = [float(x) for x in wav]
  filedat = [i.split(' ')[0] for i in filedat]
  filedat = [i[1:-1] for i in filedat]
  f.close()
  nwav = len(wav)


  nwavplot = nwav
  if (justcont == 1):
   idth = np.where(np.array(wav) == -1)[0]
   nth = np.shape(idth)[0]
   nwavplot = nwav - nth

  if (justth == 1):
   idcont = np.where(np.array(wav) != -1)[0]
   ncont = np.shape(idcont)[0]
   nwavplot = nwav - ncont

  ####!!!!       initialise the figure
  #grid spec gives better customisation of plot layout. Copied and pasted from
  #pyplot_cream_easy_2.py in mcmcmulti3 directory other codes below customise for this case






  iwavinc = [i for i in range(nwav)]
  if (justth == 1):
   iwavinc = [i for i in range(nwav) if wav[i] == -1]
  if (justcont == 1):
   iwavinc = [i for i in range(nwav) if wav[i] != -1]
  nwavinc = len(iwavinc)

  if (plots_per_page == -1):
   wavu = np.sort(np.unique(wav))
   npages = len(wavu)
  else:
   npages   = np.int(np.ceil(1.0*nwavinc/plots_per_page))


  #organise how to distribute the plots
  if (plots_per_page == -1):
   iwavinc_page = []
   for iw2 in range(npages):
    wavunow = wavu[iw2]
    iwnow = [iwavinc[i2] for i2 in range(nwavinc) if wav[iwavinc[i2]] == wavunow ]
    iwavinc_page.append(iwnow)

  elif (nwavinc > plots_per_page):
   iwavinc_page = [iwavinc[i:i+npages] for i  in range(0, len(iwavinc), npages)]

  else:
   iwavinc_page = [iwavinc]



  for ipage in range(npages):
   nwavplot = len(iwavinc_page[ipage])
   fig = plt.figure()
   if (sameplotdrive == 1 and ipage == 0):
    ipnow = 1
    gs1 = gridspec.GridSpec(nwavplot+1, 4)
    nshapegs = nwavplot+1
   else:
    ipnow = 0
    gs1 = gridspec.GridSpec(nwavplot, 4)
    nshapegs = nwavplot


   gs1.update(left=0.1, right=0.9, wspace=0.05,hspace = 0.0,bottom=0.15,top=0.99)
   ###!!!!!


   #if ipage == 0 then plot the driver up top
   if (sameplotdrive == 1 and ipage == 0):
    filedrive = '/plots/modeldrive.dat'
    datdrive = np.loadtxt(dnow + filedrive)
    ax1 = fig.add_subplot(gs1[0, 1:])
    ax1.plot(datdrive[idxinc,0], datdrive[idxinc,1],color='k',linewidth=0.5)
    ax1.fill_between(datdrive[idxinc,0],datdrive[idxinc,1]-datdrive[idxinc,2],datdrive[idxinc,1]+datdrive[idxinc,2],alpha = 0.3,color='k')
    ax1.plot(datdrive[idxinc,0], datdrive[idxinc,1]-datdrive[idxinc,2],color='k')
    ax1.plot(datdrive[idxinc,0], datdrive[idxinc,1]+datdrive[idxinc,2],color='k')
    ax1.set_xticklabels([])
    ax1.tick_params(direction='in')
    ax1.set_yticks([])
    ax1.set_xlim([tlo,thi])


   for ilc in iwavinc_page[ipage]:
    ax1 = fig.add_subplot(gs1[ipnow, 1:])
    axtf = fig.add_subplot(gs1[ipnow,:1])
    #ax1 = plt.subplot(gs1[ipnow, 1:])
    #axtf = plt.subplot(gs1[ipnow,:1])

    dat = np.loadtxt(dnow+'/../'+filedat[ilc])
    tdat = dat[:,0]
    xdat = dat[:,1]
    sigdat = dat[:,2]
    sigdat_new = np.sqrt((semed[ilc]*sigdat)**2 + vamed[ilc])
    xm = mod[:,ilc]
    sm = sigmod[:,ilc]

    #decide order of old/new error bar plotting
    for it in range(np.shape(xdat)[0]):
     signew = sigdat_new[it]
     sigold = sigdat[it]
     if (signew > sigold):
      ax1.errorbar([tdat[it]],[xdat[it]],[signew],ls='',color='b')
      if (justnewsig == 0):
       ax1.errorbar([tdat[it]],[xdat[it]],[sigold],ls='',color='r')
     else:
      if (justnewsig == 0):
       ax1.errorbar([tdat[it]],[xdat[it]],[sigold],ls='',color='r')
      ax1.errorbar([tdat[it]],[xdat[it]],[signew],ls='',color='b')



    #plot outlier rejected points if any
    drnow = self.drej[ilc]
    idrej = np.where(drnow[:,3] > 0)[0]
    ax1.scatter(drnow[idrej,0],drnow[idrej,1],color='r',marker='o')

    print(self.tmod[10],xm[10:15],ilc)
    ax1.plot(self.tmod,xm)
    ax1.fill_between(self.tmod,xm-sm,xm+sm,alpha=0.2)

    #set ylim by mod or data
    ymodlo = np.min(xdat[:]-sigdat[:])
    ymodhi = np.max(xdat[:]+sigdat[:])
    yrange = ymodhi - ymodlo
    ylim = [ymodlo-yrange/10,ymodhi+yrange/10]
    ax1.set_ylim(ylim)


    #check of we have top hat or disk response function
    if (ilc in idxth):
     axtf.hist(self.modth[:,ilc],bins=self.tau)
     thlags = np.percentile(self.modth[:,ilc],[15.865,50,84.135])
     lo = np.str(np.round(thlags[1]-thlags[0],2))
     med = np.str(np.round(thlags[1],2))
     hi = np.str(np.round(thlags[2] - thlags[1],2))
     lagtxt=r'$\tau_\mathrm{cent}='+med+'^{+'+hi+'}_{-'+lo+'}$'

     tfyl = list(axtf.get_ylim())
     axtf.set_ylim(tfyl)

     if (self.taumean == 1):
      axtf.text(0.9,0.8,lagtxt,ha='right',transform=axtf.transAxes,fontsize=8)
      axtf.plot([thlags[0]]*2,tfyl,ls=':',label=None,color='k')
      axtf.plot([thlags[1]]*2,tfyl,ls='-',label=None,color='k')
      axtf.plot([thlags[2]]*2,tfyl,ls=':',label=None,color='k')
    else:
     #disk response function
     #response function only saved once in wavelength file
     #if have multiple lc's at same wavelength, need to plot the response function
     #from the first light curve at that wavelength
     wavnow = wav[ilc]
     idsamewav = np.where(np.array(wav) == wavnow)[0]
     if (np.shape(idsamewav)[0] > 0):
      itf = idsamewav[0]
     else:
      itf = ilc
     axtf.plot(self.tau,self.modtf[:,itf])
     axtf.fill_between(self.tau,self.modtf[:,itf]-self.sigtf[:,itf],self.modtf[:,itf]+self.sigtf[:,itf],alpha=0.2)
     yltf = list(axtf.get_ylim())
     if (taumeanplot == 1):
      taumean = np.sum(self.tau*self.modtf[:,itf])/np.sum(self.modtf[:,itf])
      axtf.plot([taumean]*2,yltf,color='k')
     if (tau90plot > 0):
      Ntau = np.shape(self.tau)[0]
      psi90 = tau90plot
      psimax = np.max(self.modtf[:,itf])
      for itau in range(Ntau-1,0,-1):
       psinow = self.modtf[itau,itf]/psimax
       if (psinow > psi90):
        tau90 = self.tau[itau]
        id90 = itau
        break
      axtf.plot([self.tau[0],self.tau[-1]],[psi90]*2,color='r',ls='--')
      #axtf.plot([self.tau90]*2,yltf,color='r',ls='--')
      if (tauplot0 == 1):
       axtf.plot([self.tau[0],self.tau[-1]],[0]*2,color='k')
    #miscellaneous label positions/tick mark info etc
    if (ipnow == nshapegs - 1):
     axtf.set_xlabel(self.xtflab)
     ax1.set_xlabel(self.xlclab)
    else:
     ax1.set_xticklabels([])
     axtf.set_xticklabels([])
    if (ipnow == nshapegs/2):
     axtf.set_ylabel('response (relative units)')
     ax1.set_ylabel('flux (relative units)')





    ax1.tick_params(direction='in')
    axtf.set_yticklabels([])
    axtf.set_yticks([])
    ax1.set_yticks([])
    ax1.yaxis.set_label_position('right')
    ax1.set_yticklabels([])

    if (forcelag != []):
     axtf.set_xlim(forcelag)

    if (forcelab == []):
     ax1.text(0.9,0.8,filedat[ilc],ha='right',transform=ax1.transAxes,fontsize=8)
    elif (forcelab == 0):
     pass
    else:
     ax1.text(0.9,0.8,forcelab[ilc],ha='right',transform=ax1.transAxes,fontsize=8)

    #title
    if (ipnow == 0):
     ax1.text(0.5,1.2,tit[idnow],ha='center',transform=ax1.transAxes,fontsize=14)

    ax1.set_xlim([tlo,thi])
    ipnow = ipnow + 1

   ax1.set_title(self.head[idnow])
   fig_objects.append(fig)
  return(fig_objects)
   #plt.savefig('page_'+np.str(np.int(ipage))+'_'+'lcplot_'+np.str(tit[idnow])+'.pdf')



 def plot_posterior(self):
  '''
  plot the disk posterior covariances
  :return:
  '''
  dnow = self.dnow
  head = self.header
  extents = self.extents
  tit = self.title
  true = self.true
  #fsave = 'posterior_'+tit[idnow]+'.pdf'
  fsave = ''

  try:
   fig, axes = cpos.cream_posterior(dnow,true=true,header=head[idnow],extents_in=extents,fsave=fsave)
  except:
   print('unable to make covariance plot for disc posteriors. Please check at least some of these are set to vary'
         'in the fit.')
   fig = None
   axes = None

  return(fig,axes)



# script to convert all files of type PS in directory to png

import numpy as np
import glob
import os


def myps2png(dir):


 pwd = os.getcwd()
 os.chdir(dir)
 fname = glob.glob('*.PS')
 nf = len(fname)


 for i in range(nf):
  file = fname[i]
  idxext  = file.find('.')
  com = 'convert -alpha off '+file+' '+file[:idxext]+'.png'
  os.system(com)
 os.chdir(pwd)


import numpy as np
from scipy.stats import gaussian_kde



def kde_max(kde, tol=1e-3):
    """Determine the maximum value of a KDE
    INPUT: scipy.stats.kde object"""
    # 2010-07-23 11:14 IJC: Created
    cenguess = np.median(kde.dataset,1)
    init_steps = np.std(kde.dataset,1)
    max_vals = np.zeros(kde.d)
    for dim in range(kde.d):
        this_step = init_steps[dim].copy()
        tempkde = gaussian_kde(kde.dataset[dim,:])
        this_cen = cenguess[dim].copy()
        dguess = 1.
        while dguess>tol:
            pos_guess = this_cen + this_step
            neg_guess = this_cen - this_step
            if tempkde(pos_guess)>tempkde(this_cen):
                dguess = abs(this_cen-pos_guess)
                this_cen = pos_guess
            elif tempkde(neg_guess)>tempkde(this_cen):
                dguess = abs(this_cen-neg_guess)
                this_cen = neg_guess
            else: 
                this_step *= 0.5
                dguess *=0.5
        max_vals[dim] = this_cen
    return max_vals


def findkdeval(kde, val, guess=None,tol=1e-6, maxiter=1000,verbose=False):
    """Find the "x"-value such that kde(x)=val +/- tol

    Without a specified guess for a monomodal distribution, tends to
    find the lower of the two possible values.

    Uses scipy.stats.gaussian_kde objects"""
    # 2010-07-23 11:49 IJC: Created
    if maxiter is None:
        maxiter = 1000
    
    if kde.d>1:
        retvals = np.zeros(kde.d,float)
        if hasattr(val,'__iter__'):
            pass
        else:
            val = np.tile(val,kde.d)

        for dim in range(kde.d):
            print dim
            tempkde = gaussian_kde(kde.dataset[dim,:])
            retvals[dim] = findkdeval(tempkde, val[dim],tol=tol, guess=guess,maxiter=maxiter)
    else:
        max_location = kde_max(kde)
        maxval = kde(max_location)
        if val>maxval:
            print "Value entered (%01.3g) is greater than KDE's maximum value (%01.3g)" % (val,maxval)
            return max_location
        
        iter = 0
        initial_step = np.std(kde.dataset)
        step = initial_step
        medval = np.median(kde.dataset)
        if guess is None:
            guess = medval - step
        else:
            pass

        initial_guess = guess # Save this, just in case
        kdeval = kde(guess)
        dval = abs(kdeval-val)
        while dval>tol and iter<=maxiter:
#            pos = min(guess + step, max_location)
            pos = guess + step
            neg = guess - step
            kdepos = kde(pos)
            kdeneg = kde(neg)
            if verbose and (iter/10)==(iter/10.):
                print "iter, val, pos, neg, kdepos,kdeneg,guess, step"
            if verbose:
                print "%i %01.6g %01.6g %01.6g %01.6g %01.6g %01.6g" %(iter, val, pos, neg,kdepos,kdeneg, step)
            dval_pos = abs(kdepos-val)
            dval_neg = abs(kdeneg-val)
            if dval_pos<dval: # we're closer than we were!
                dval = dval_pos
                guess = pos
            elif dval_neg<dval: # we're closer than we were!
                dval = dval_neg
                guess = neg
            elif dval_pos==dval_neg: # We must be really far away
                if initial_guess>medval:
                    guess = medval + initial_step
                else:
                    guess = medval - initial_step
            else: # neither guess is closer, but we're still not within tol
                step *= 0.5

            iter += 1
        retvals = guess
        if iter>maxiter:
            print "Exceeded maximum number of iterations (%i) without convergance" % maxiter

    return retvals
                
            
 


def conflevel(kde, frac, ftol=1e-6, tol=1e-6, usespline=False, verbose=False,maxiter=None):
    """Determine the lower and upper confidence levels required to
    enclose a given fraction 'frac' of a KDE object's dataset(s) to
    within a tolerance ftol."""
    # 2010-07-23 14:00 IJC: Created
    from scipy import interpolate

    if kde.d>1:
        ret = []
        for ii in range(kde.d):
            if verbose: print ii,'/',kde.d
            ret.append(conflevel(gaussian_kde(kde.dataset[ii]),frac,ftol=ftol,tol=tol,usespline=usespline,verbose=verbose,maxiter=maxiter))
        return ret
    else:
        median_value = np.median(kde.dataset)
        step = np.std(kde.dataset)
        low_limit = median_value - 2*step
        enclosed_fraction = np.Inf
        thismovewasup=True
        if usespline:
            spx = np.linspace(-5*step,5*step,1e3)+median_value
            sp = interpolate.UnivariateSpline(spx,kde(spx),k=3.0,s=0.0)
            sp.d = kde.d
            sp.dataset = kde.dataset
            kde0 = kde
            kde = sp

        while abs(enclosed_fraction-frac)>ftol:
            kdelow = kde(low_limit)
            guess = 2*median_value-low_limit
            if verbose:
                print "kdelow,guess",kdelow,guess
            high_limit = findkdeval(kde, kdelow, tol=tol, guess=guess,verbose=verbose,maxiter=maxiter)
            if usespline:
                enclosed_fraction = kde0.integrate_box_1d(low_limit, high_limit)
            else:
                enclosed_fraction = kde.integrate_box_1d(low_limit, high_limit)

            lastmovewasup = thismovewasup
            if enclosed_fraction > frac: # need to close in
                low_limit += step
                thismovewasup = True
            else: # need to open up limits
                low_limit += -step
                thismovewasup = False

            movingInSameDirection = lastmovewasup==thismovewasup
            if movingInSameDirection: # keep going
                pass
            else: # We passed it; turn around and home in
                step *= 0.5

    return low_limit, high_limit



def confmap(map, frac, **kw):
    """Return the confidence level of a 2D histogram or array that
    encloses the specified fraction of the total sum.

    :INPUTS:
      map : 1D or 2D numpy array
        Probability map (from hist2d or kde)

      frac : float, 0 <= frac <= 1
        desired fraction of enclosed energy of map

    :OPTIONS:
      ordinate : None or 1D array
        If 1D map, interpolates onto the desired value.  This could
        cause problems when you aren't just setting upper/lower
        limits....
    """
    # 2010-07-26 12:54 IJC: Created
    # 2011-11-05 14:29 IJMC: Fixed so it actually does what it's supposed to!
    from scipy.optimize import bisect

    def diffsum(level, map, ndesired):
        return ((1.0*map[map >= level].sum()/map.sum() - ndesired))

    if hasattr(frac,'__iter__'):
        return [confmap(map,thisfrac, **kw) for thisfrac in frac]

    #nx, ny = map.shape
    #ntot = map.size
    #n = int(ntot*frac)

    #guess = map.max()
    #dx = 10.*float((guess-map.min())/ntot)
    #thisn = map[map<=guess].sum()

    ret = bisect(diffsum, map.min(), map.max(), args=(map, frac))
    if kw.has_key('ordinate') and kw['ordinate'] is not None:
        sortind = np.argsort(map)
        ret = np.interp(ret, map[sortind], kw['ordinate'][sortind])

    return ret



def kdehist2(x, y, npts, xrange=None, yrange=None):
    """Generate a 2D histogram map from data, using Gaussian KDEs

    :INPUTS:
      x : seq
        X data

      y : seq
        Y data

      npts : int or 2-seq
        number of points across final histogram, or [nx, ny]

    :OPTIONAL_INPUTS:
      xrange : 2-seq
        [x_min, x_max] values for final histogram

      yrange : 2-seq
        [y_min, y_max] values for final histogram
        
    :RETURNS:
      [kdehist, xbins, ybins]

    :EXAMPLE:
      ::
 
        import kdestats as kde
        import numpy as np
        import pylab as py

        covmat = [[1., 1.5], [1.5, 4.]]
        xy = np.random.multivariate_normal([0, 0], covmat, [1e4])
        kdehist = kde.kdehist2(xy[:,0], xy[:,1], [30, 30])
        clevels = kde.confmap(kdehist[0], [.6827,.9545,.9973])

        py.figure()  # Plot 1-, 2-, and 3-sigma contours
        c = py.contour(kdehist[1], kdehist[2], kdehist[0], clevels)
    """
    # 2012-02-11 19:45 IJMC: Created

    if hasattr(npts, '__iter__'):
        if len(npts)==1:
            npts = [npts[0], npts[1]]
    else:
        npts = [npts, npts]

    # Generate KDE:
    thiskde = gaussian_kde([x, y])

    # Generate coordinates for KDE evaluation:
    if xrange is None:
        thisx0 = np.median(thiskde.dataset[0])
        thisdx0 = np.std(thiskde.dataset[0])
        thisx = np.linspace(-5*thisdx0,5*thisdx0,npts[0])+thisx0
    else:
        thisx = np.linspace(xrange[0], xrange[1], npts[0])

    if yrange is None:
        thisy0 = np.median(thiskde.dataset[1])
        thisdy0 = np.std(thiskde.dataset[1])
        thisy = np.linspace(-5*thisdy0,5*thisdy0,npts[1])+thisy0
    else:
        thisy = np.linspace(yrange[0], yrange[1], npts[1])

    thisxx,thisyy = np.meshgrid(thisx,thisy)
    thishist = thiskde([thisxx.ravel(),thisyy.ravel()]).reshape(npts[0],npts[0])

    return thishist, thisx, thisy




#insert fluxes and wavelengths
#deredden these to nu^1/3 and report back E(B-V) gaskell 2004

import numpy as np
import myextmag as em
import matplotlib.pylab as plt
import scipy.optimize as so
import myrandom as mr

wav = np.arange(2000,9000,1000.)
nw = np.shape(wav)[0]
wav0 = 4000.0
fnu0 = 10.0

ebmvtrue = 0.16
A0 = em.extmag_agn(wav0,1.0)[0]
Awav = em.extmag_agn(wav,1.0)
Awav = np.array(Awav)
fnu = fnu0*(wav/wav0)**(-1./3)*10**(-0.4*ebmvtrue*(Awav)) 


#fnu[0:3] = fnu[0:3] - 5.0

sigfnu = np.ones(nw)
fnu = fnu + mr.normdis(1,0.,sigfnu)

logfnu = np.log10(fnu)
logwav = np.log10(wav)

rln10 = np.log(10.)
siglog = 1./rln10/fnu * sigfnu





nw = wav.shape[0]
#wav0 = wav[nw/2]

#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! define function to fit!!!!!!!!!!!!!!!!!!!!!!!
#wav_wav0_log = log(wav/wav0)
def fnu_red_log(wav_wav0_log,fnu0_log,ebmv): 
 return(fnu0_log - 0.33333*wav_wav0_log - 0.4*ebmv*(Awav))
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 


wav_wav0_log = np.log10(wav/wav0)

#now fit the reddening function (Get back ebmv).
fit_coef = so.curve_fit(fnu_red_log,wav_wav0_log,logfnu,sigma=siglog)
sig_coef=np.sqrt([fit_coef[1][0,0],fit_coef[1][1,1]])
Nres = 100

wavhi = wav[-1]*1.3
wavlo = wav[0]*0.7
wavmod = wavlo + 1.*np.arange(Nres)/Nres*(wavhi-wavlo)
logwavmod=np.log10(wavmod)

fnu0_log = fit_coef[0][0]
sigfnu0_log = sig_coef[0]
sigebmv     = sig_coef[1]
ebmv     = fit_coef[0][1]
Awavmod = np.array(em.extmag_agn(wavmod,1.0))
logfnumod = fnu0_log - 0.33333*(logwavmod - np.log10(wav0)) #- 0.4*ebmv*(Awavmod )
siglogfnu = np.sqrt(sigfnu0_log**2 + (0.4*(Awavmod-A0)*sigebmv)**2) 

rln10 = np.log(10.)
lfnumod10 = 10**logfnumod

fnumod = lfnumod10*(1. + 0.5*rln10*rln10*siglogfnu*siglogfnu)
sigfnumod = rln10*lfnumod10*siglogfnu
#


#plot the result

fig=plt.figure()
ax1=fig.add_subplot(111)
ax1.errorbar(wav,fnu,sigfnu,linestyle='',color='r',linewidth=4)
fnumodlo = fnumod-sigfnumod
fnumodhi = fnumod+sigfnumod
ax1.plot(10**logwavmod,fnumodhi,color='r')
ax1.plot(10**logwavmod,fnumodlo,color='r')
ax1.fill_between(10**logwavmod,fnumodlo,fnumodhi,color='r',alpha=0.4)

l=[r'$ E(B-V)='+str(round(ebmv,2))+' \pm '+str(round(sigebmv,2))+'$']
ax1.plot(10**logwavmod,fnumod,color='r',label=l[0])
ax1.legend(l)

ax1.set_title('Mean and Variable Component Spectrum')   
ax1.set_xscale('log')
ax1.set_yscale('log')

ax1.set_xlim(wavlo,wavhi)
ax1.set_xlabel(r'$ \mathrm{Wavelength} / \AA$')
ax1.set_ylabel(r'$f_{\nu} \left( \lambda \right)$')
xtl = np.arange(wavlo+(1000.-np.mod(wavlo,1000.)),wavhi-np.mod(wavhi,1000),2000.)#np.logspace(np.log10(wavlo),np.log10(wavhi),5)#[100,200,500,1000]
ax1.set_xticks(xtl)
xtls = ["%d" % f for f in xtl]
ax1.set_xticklabels(xtls)


plt.show()
 
 















import numpy as np

#define function to convert sdss magnitudes to panstarrs magnitudes (from Morganson et al 2014)
#input gmag_sub_imag g magnitude minus i magnitude
#MUst have g and i band magnitudes in sdss

def diff_magps(gmag_sub_imag):
 #gmagsdss = magsdss[0]
 #imagsdss = magsdss[1]
 a0 = np.array([0.00128,-0.00518,0.00585,0.00144])
 a1 = np.array([-0.10699,-0.03561,-0.01287,0.07379])
 a2 = np.array([0.00392,0.02359,0.00707,-0.0336])
 a3 = np.array([0.00152,-0.00447,-0.00178,0.00765])
 gi = 1.*gmag_sub_imag#gmagsdss - imagsdss
 mpsout = a0 + a1*gi + a2*gi**2 + a3*gi**3
 return(mpsout)
 
 
#alternative approach by finkbeiner + 2015 now need difference in panstarrs g and i as input 
#this one has ugrizy filters
def diff_magps_fink(gmag_sub_imag):
 #gmagps = magps[0]
 #imagps = magps[1]
 a0 = np.array([0.04438,-0.01808,-0.01836,0.01170,-0.01062,0.08924])
 a1 = np.array([-2.26095,-0.13595,-0.03577,-0.00400,0.07529,-0.20878])
 a2 = np.array([-0.13387,0.001941,0.02612,0.00066,-0.03592,0.10360])
 a3 = np.array([0.27099,-0.00183,-0.00558,-0.00058,0.00890,-0.02441])
 gi = 1.*gmag_sub_imag#gmagsdss - imagsdss
 mpsout = a0 + a1*gi + a2*gi**2 + a3*gi**3
 return(mpsout)
 
 
 
 
 
 
def sdss_mag_2_flux(mag,sigmag,filter):
 
 if (filter == 'u'):
  b =1.4e-10
 elif (filter == 'g'):
  b = 0.9e-10
 elif (filter == 'r'):
  b = 1.2e-10
 elif (filter == 'i'):
  b = 1.8e-10
 elif (filter == 'z'):
  b = 7.4e-10
 
 b2 = 2*b
 a = (-np.log(10)/2.5*mag - np.log(b))#*2*b
 siga = np.abs(-np.log(10.)/2.5*sigmag)
  
 
 f_f0 = np.sinh(a)*b2
 sig_f_f0 = np.abs(np.cosh(a)*siga)*b2
 
 flux = 3631.*10**6 * f_f0
 sigflux = 3631*10**6 * sig_f_f0
 return(flux,sigflux)


# python code to produce an optimal average
#is spec = 1 then avoid points with zero error bars
import numpy as np


def myoptaverage(yin,sigyin,spec=1):
    
    ny = np.shape(yin)[0]
    if (spec == 1):
     idx = np.where(sigyin != 0)[0]
    else:
     idx = np.arange(ny)
    
    y = yin[idx]
    sigy = sigyin[idx]
    
    sigy2=sigy*sigy
    
    top=(y/sigy2).sum()
    bot=(1/sigy2).sum()
    optav=top/bot
    
    sig = np.sqrt(1/bot)
    return(optav,sig)
    



#python project to load in numbers spreadsheets for each 
#exp_xxxx.numbers file in /Users/david/projects/expenses_data
#seasonal arima model article https://www.analyticsvidhya.com/blog/2018/02/time-series-forecasting-methods/

import numpy as np
import matplotlib.pylab as plt
import glob
import pandas as pd
import datetime
import statsmodels.api as sm
import vortexa_convert_df2lc as df2lc
import vortexa_polyfit as vp

#a few notes on the final argument of seasonal order
#e.g 120. This can tak evalues up to the size of the input time series
# Higher numbers will encourage the model to search longer into the past for
# seasonal variability at the expense of computation time
# if too small a number is chosen here, the model (although faster) may fail to capture
# seasonal variations

def sarima(lc1,orderin=(1,1,0),seasonal_order = 'auto',trend='c',enforce_invertibility=False,
alpha_confidence=0.32,time_forecast=10.0,diagnostic_plot=''):




 if (type(lc1) == np.ndarray):
  time   = lc1[:,0]
  signal = lc1[:,1]
  print ('sarima input is numpy array')
 elif (type(lc1) == pd.core.frame.DataFrame):
  print('sarima input is a data frame')
  a = df2lc.convert_df_2_lightcurve(lc1,reftime='auto',noise = 0.1,normalise = 0)
  print(a)
  time = a[:,0]
  signal = a[:,1]
 
 print(type(lc1))
 
 
 dt = np.mean(time[1:]-time[:-1])
 nforecast = np.int(time_forecast/dt)
 ntime = np.shape(time)[0]
 tfclo = time[ntime-1]
 tfchi = tfclo + nforecast*dt
 idx_forecast_lo = ntime - 1
 idx_forecast_hi = ntime + nforecast
 
 
 #set up the model parameters
 if (seasonal_order == 'auto'):
  so_final = np.int(0.70*ntime)
  so = (0,1,0,so_final)
 else:
  so = seasonal_order
 
 #fit the sarima model
 model   = sm.tsa.statespace.SARIMAX(endog=signal,
 order=orderin,
 seasonal_order=so,
 trend='c',
 enforce_invertibility=False)
 
 for i in range(ntime):
  print(i,time[i],signal[i])
 print('idx fc ',idx_forecast_lo,idx_forecast_hi)
 results = model.fit()
 pred    = results.get_prediction(start = ntime, end= idx_forecast_hi )
 ps      = pred.summary_frame(alpha=alpha_confidence)
 pslo    = np.array(ps['mean_ci_lower'])
 pshi    = np.array(ps['mean_ci_upper'])
 npred   = np.shape(pslo)[0]
 


 #output the forecast time series
 ymean = np.array(ps['mean'])
 ylo  = np.array(ps['mean_ci_lower'])
 yhi  = np.array(ps['mean_ci_upper'])
 nym = np.shape(ymean)[0]
 xmod = np.linspace(tfclo,tfchi,nym)

  

 #plot the time series 9can also use dweek.plot() for simple option but less customisation
 if (diagnostic_plot != ''):
  x = time
  y = signal
  fig = plt.figure()
  ax1 = fig.add_subplot(111)
  ax1.plot(x,y,label='data')
  ax1.plot(xmod,ymean,label='forecast')
  ax1.set_title(lab)
  ax1.fill_between(xmod,ylo,yhi,alpha = 0.3,label='uncertainty')
  ax1.set_xlabel('Time')
  ax1.set_ylabel('light curve')
  plt.legend(fontsize='xx-small')
  if (disgnostic_plot != 'show'):
   plt.savefig(diagnostic_plot)
   plt.clf()
  else:
   plt.show()
   
 #if the input was a data frame convert the time axis back into datetime format
 if (type(lc1) == pd.core.frame.DataFrame): 
  date_start = lc1.values[0,0]
  date_final = lc1.values[-1,0]
  time_out = np.array( [date_final + datetime.timedelta(days=xm-xmod[0]) for xm in xmod] )
 else:
  time_out = xmod 
   
   
   
   
   
   
 #compute the model Mean Absolute Percentage Error (MAPE) using cross validation
 #extract a smaller time series (smaller by length equal to the forecast)
 #use cross validation to work out the MAPE.
 frac_check = 0.9
 id_check = np.int(frac_check*ntime)
 model_check = sm.tsa.statespace.SARIMAX(endog=signal[:id_check],
 order=orderin,
 seasonal_order=so,
 trend='c',
 enforce_invertibility=False,
 enforce_stationarity=False)
 results_check = model_check.fit()
 pred_check = results_check.get_prediction(start = 0, end=ntime - 1 )
 ps_check      = pred_check.summary_frame(alpha=alpha_confidence)
 y_check = np.array(ps_check['mean'])
 
 print(id_check,ntime)
 print(y_check)
 print(signal[id_check:])
 
 mape = np.sum( np.abs(y_check[id_check:] - signal[id_check:])/signal[id_check:] )/(ntime-id_check)
 

 
 #import matplotlib.pylab as plt
 
 #plt.clf()
 #plt.plot(signal)
 #plt.plot(y_check)
 #plt.show()
 #print(mape,ntime-id_check)
 #input() 
 
 
 return(time_out,ymean,ylo,yhi,mape)
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
#  sarima(lc1,orderin=(1,1,0),seasonal_order = 'auto',trend='c',enforce_invertibility=False,
#alpha_confidence=0.32,time_forecast=10.0,diagnostic_plot=''):
#time_forecast argument should be in days not units
 

def polyfit(lc1,order='test',time_forecast=10.0,alpha_confidence=0.32,frac_check=0.9):
 
 
 if (type(lc1) == np.ndarray):
  time   = lc1[:,0]
  signal = lc1[:,1]
  print ('sarima input is numpy array')
 elif (type(lc1) == pd.core.frame.DataFrame):
  print('sarima input is a data frame')
  a = df2lc.convert_df_2_lightcurve(lc1,reftime='auto',noise = 0.1,normalise = 0)
  print(a)
  time = a[:,0]
  signal = a[:,1]
 
 print(type(lc1))
 
 
 dt = np.mean(time[1:]-time[:-1])
 nforecast = np.int(time_forecast/dt)
 ntime = np.shape(time)[0]
 tfclo = time[ntime-1]
 tfchi = tfclo + nforecast*dt
 idx_forecast_lo = ntime - 1
 idx_forecast_hi = ntime + nforecast
  
 #grid search to find the optimum number of parameters
 if (order == 'test'):
  best_cisqred,best_aic,best_bic = vp.fit_search(time,signal,maxorder=6)
  idx = best_cisqred
 else:
  idx = order
  
 
 #perform the fit
 time_plus_fc = np.arange(time[0],time[-1]+time_forecast+dt,dt)
 yg_med,yg_lo,yg_hi,cov,cisq,cisq_red,bic,aic = \
 vp.fit(time,signal,np.ones(ntime),order=3,xgrid=time_plus_fc,confidence=alpha_confidence,nits=2000)
 
 
 
 
 #if the input was a data frame convert the time axis back into datetime format
 if (type(lc1) == pd.core.frame.DataFrame): 
  date_start = lc1.values[0,0]
  date_final = lc1.values[-1,0]
  time_out = np.array( [date_final + datetime.timedelta(days=xm-time[-1]) for xm in time_plus_fc[ntime:]] )
 else:
  time_out = time_plus_fc[ntime:]
   
    
 y_out   = yg_med[ntime:]
 ylo_out = yg_lo[ntime:]
 yhi_out = yg_hi[ntime:]
 
 
 
 
 
 #evaluate the mape
 ncheck = np.int(frac_check*ntime)
 time_check_fc = np.arange(time[0],time[-1]+dt,dt)
 check_med,check_lo,check_hi,check_cov,check_cisq,check_cisq_red,check_bic,check_aic = \
 vp.fit(time[:ncheck],signal[:ncheck],np.ones(ncheck),order=idx,xgrid=time,confidence=alpha_confidence,nits=2000)
 mape = np.sum( np.abs( (signal[ncheck:] - check_med[ncheck:])/signal[ncheck:]) )/ (ntime-ncheck + 1) 
  
  
 
 
 
 
 
 return(time_out,y_out,ylo_out,yhi_out,mape)
 















# python code to take an input light curve add signal to noise and save resave
import myrandom
import numpy as np

#inputs:  Dat 2d array of x,y,sig,
#         SNR desired signal to noise ratio
#         File output filename
# output: the modified data with altered with gaussian noise with the input snr

def myaddnoise(dat,snr):
    x   = dat[:,0]
    y   = dat[:,1]
    sig = dat[:,2]
    
    
    signew=np.abs(y/snr)
    ynew=myrandom.normdis(1,y,signew)
    datnew=np.ones((dat[:,0].shape[0], 3))
    datnew[:,0]=x
    datnew[:,1]=ynew
    datnew[:,2]=signew
    
    
    #for i in range(datnew[:,0].shape()[0]):
    # print datnew[i,0],datnew[i,1],datnew[i,2]
     
     
    return(datnew)    
    
    
    
    



import numpy as np
import pylab as plt



#script to read an input file plot histograms of the data
#sublabfile if entered are the labels to label each of the plot windows
#xlim if entered are the limits to force all the histograms to lie within

def multihist(filename,nperhist,nvert,nalong,parcol,sublabfile = '',xlim=[], bin1=50):

 
 if (nperhist > 1):
  translucency = 0.6
 else:
  translucency = 0.0
 
 with file(filename) as f:
    fname = f.read().splitlines()
    
 nf  = len(fname)


 nsublab = len(sublabfile)
 if (nsublab > 0):
  with file(sublabfile) as f:
   sublab = f.read().splitlines()  
 


 
 dat=[]
 #!load the data
 for i in range(nf):
  print fname[i],i
  a   = np.genfromtxt(fname[i],skip_footer=1,usecols = np.arange(5))
  dat.append(a)
  #print a
  
#! plot the data
 fig = plt.figure()

#! alter python plotting to plot in column rather than row order 
 ic = 0
 isp = np.arange(nvert*nalong) + 1
 isp = np.reshape(isp,(-1,nalong))
 isp = np.reshape(isp,nvert*nalong,order='F')
 
 for i in range(nvert*nalong):
  ax1 = fig.add_subplot(nvert,nalong,isp[i])
    
  for i2 in range(nperhist):
   ax1.hist(dat[ic][:,parcol], edgecolor = 'None', alpha = translucency,bins = bin1)
   ic = ic + 1

 #label the subplots if file is provided
  if (nsublab > 1):
   
   a = np.array(ax1.axis()) #expand yaxis to make room for lab
   a[-1] = a[-1]*1.5
   ax1.axis(a)
   ax1.text(0.1,0.85,sublab[i],ha='left',va='top',transform=ax1.transAxes,fontsize = 12)
   
  #force the x axis limits to all be the same if entered
  if (len(xlim) > 0):
   a = np.array(ax1.axis())
   a[[0,1]] = xlim[0],xlim[1]
   ax1.axis(a)
   
  ax1.set_yticklabels('')
  if (np.mod(i+1,nvert) != 0):
   ax1.set_xticklabels('')



##script to test the above definition
#
#nf = 16
#nalong = 2
#nvert = 4
#parcol = 0
#nperhist = 2
#
#fnam = 'histmultidat.dat'
#f = open(fnam,'w')
#for i in range(nf):
# fname = 'test_multihis_'+str(i)+'.dat'
# f.write(fname+'\n')
# a = np.random.randn(1000)
# np.savetxt(fname,a)
#f.close()
#
#
#
#
#a = multihist(fnam,nperhist,nvert,nalong,parcol,sublabfile = fnam,xlim =[-5,5] )
#
#plt.savefig('myhistmultitest.png')
 

 
 



import numpy as np


def myredcisq(tdat,xdat,sigdat,tmod,xmod,npar = 0):

 ndat = np.shape(tdat)[0]
 xint = np.interp(tdat,tmod,xmod)
 cisq = np.sum( ((xint - xdat)/sigdat)**2)
 redcisq = cisq/(ndat-npar)
 
 return(cisq,redcisq)


#NOTE when rebinning if you have a SNR in mind, remember what cadence the SNR applies to. i.e ~500 for 0.5 day sampled kepler light curves. If you have finely sampled data and you bin it up. SNR will improve. 
#program to take an input time series and rebin it according to the parameters in the input
# rebin optimally averages all data within the new cadence range and caluclates errors according to keith ada notes lecture 4
import numpy as np
import os
import sys


def myrebin(dir,files,cadence):
 #dir=sys.argv[1]       #the directory storing the light curves e.g '../fort/fortcode/fakelc/kep_18mar'
 #files = sys.argv[2]   # a list of the files .e.g ['file1.dat','file2.dat','...'] etc
 #cadence = sys.argv[3] #e.g 0.5 will rebin into half day cadence
 
 
 #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 3 user inputs defined above!!!!!!!!!!!!!!!!!
 
 
 dirpy = os.getcwd()
 os.chdir(dir)
 
 dirreal = os.getcwd()
 
 nfiles=len(files)
 
 
 #fname=raw_input('Please enter file name:-')
 for ifile in range(nfiles):    
     fname=files[ifile]
     dat=np.loadtxt(fname) #load the data
     
     t=dat[:,0]
     x=dat[:,1]
     sig=dat[:,2]
     ndat=t.shape[0]
     trange=t[-1]-t[0]
     dtbefore=trange/(ndat-1)
     nbin=int(np.ceil(trange/cadence))
     
     op=np.zeros((nbin,3))
     
      
     for i in range(nbin):
         binlo=t[0]+i*cadence
         binhi=binlo+cadence
         binmid=binlo+cadence/2
         idx=np.where((t>=binlo) & (t<binhi))[0] # array of indices of all elements to go in this bin
         bintime = np.median(t[idx])
         
         op[i,0]=bintime
         top=(x[idx]/sig[idx]**2).sum()
         bot=(1./sig[idx]**2).sum()
         op[i,1]=top/bot
         sigbin=1/np.sqrt(bot)
         op[i,2]=sigbin
         
     print 'consider your data re-binned!' 
     
 #    os.chdir(path)
     op=op[np.where(op[:,1] == op[:,1])[0],:]
     
     #os.chdir(dirnew)
     np.savetxt('rebinned_'+str(fname),op)
     #os.chdir(dirreal)
    
 os.chdir(dirpy)
 return()


# test_therm_plot scripr
import numpy as np
import pylab as plt
import scipy.optimize


function myplotxyfitline(fname,x,y,sig):




    dat=np.loadtxt(fname)

    x=dat[:,0]
    y=dat[:,1]
    n=y.shape[0]
    nmod=n*100


### analyse data
    def ymod(x,a,b):
        ym=a+b*x
        return(ym)
###


# analyse slope
    fit_coeff=scipy.optimize.curve_fit(ymod,x,y,sigma=sig)

    coeff_err=[fit_coeff[1][0,0],fit_coeff[1][1,1]]
    fit_coeff=fit_coeff[0]

    xmod=np.arange(x[0],x[-1],(x[-1]-x[0])/nmod)
    ymod=fit_coeff[0]+fit_coeff[1]*xmod
##



#plot log slope

    fig=plt.figure()
    ax1=fig.add_subplot(1,1,1)
    ax1.errorbar(x,y,sig)
    ax1.plot(xmod,ymod)
    plt.title('Something not very useful')

    ax1.set_xlabel('x')
    ax1.set_ylabel('y')
    ax1.text(0.5,0.5,'ic='+str(round(fit_coeff[0],2))+'+/-'+str(round(coeff_err[0]))+'  m='+str(round(fit_coeff[0],2))'+/-'+str(round(coeff_err[1],2)),ha='left',transform=ax1.transAxes)

    plt.show()


